@@ -9,8 +9,8 @@
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
 //
-// The above copyright notice and this permission notice shall be included in all
-// copies or substantial portions of the Software.
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -25,22 +25,17 @@
 // HIP API
 #include "hipcub/block/block_histogram.hpp"
 
-
 #ifndef DEFAULT_N
 const size_t DEFAULT_N = 1024 * 1024 * 128;
 #endif
 
-template<
-    class Runner,
-    class T,
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    unsigned int BinSize,
-    unsigned int Trials
->
-__global__
-__launch_bounds__(BlockSize)
-void kernel(const T* input, T* output)
+template<class Runner,
+         class T,
+         unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int BinSize,
+         unsigned int Trials>
+__global__ __launch_bounds__(BlockSize) void kernel(const T* input, T* output)
 {
     Runner::template run<T, BlockSize, ItemsPerThread, BinSize, Trials>(input, output);
 }
@@ -48,18 +43,15 @@ void kernel(const T* input, T* output)
 template<hipcub::BlockHistogramAlgorithm algorithm>
 struct histogram
 {
-    template<
-        class T,
-        unsigned int BlockSize,
-        unsigned int ItemsPerThread,
-        unsigned int BinSize,
-        unsigned int Trials
-    >
-    __device__
-    static void run(const T* input, T* output)
+    template<class T,
+             unsigned int BlockSize,
+             unsigned int ItemsPerThread,
+             unsigned int BinSize,
+             unsigned int Trials>
+    __device__ static void run(const T* input, T* output)
     {
         const unsigned int index = ((hipBlockIdx_x * BlockSize) + hipThreadIdx_x) * ItemsPerThread;
-        unsigned int global_offset = hipBlockIdx_x * BinSize;
+        unsigned int       global_offset = hipBlockIdx_x * BinSize;
 
         T values[ItemsPerThread];
         for(unsigned int k = 0; k < ItemsPerThread; k++)
@@ -67,18 +59,19 @@ struct histogram
             values[k] = input[index + k];
         }
 
-        using bhistogram_t =  hipcub::BlockHistogram<T, BlockSize, ItemsPerThread, BinSize, algorithm>;
-        __shared__ T histogram[BinSize];
+        using bhistogram_t
+            = hipcub::BlockHistogram<T, BlockSize, ItemsPerThread, BinSize, algorithm>;
+        __shared__ T                                  histogram[BinSize];
         __shared__ typename bhistogram_t::TempStorage storage;
 
-        #pragma nounroll
+#pragma nounroll
         for(unsigned int trial = 0; trial < Trials; trial++)
         {
             bhistogram_t(storage).Histogram(values, histogram);
         }
 
-        #pragma unroll
-        for (unsigned int offset = 0; offset < BinSize; offset += BlockSize)
+#pragma unroll
+        for(unsigned int offset = 0; offset < BinSize; offset += BlockSize)
         {
             if(offset + hipThreadIdx_x < BinSize)
             {
@@ -89,49 +82,44 @@ struct histogram
     }
 };
 
-template<
-    class Benchmark,
-    class T,
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    unsigned int BinSize = BlockSize,
-    unsigned int Trials = 100
->
+template<class Benchmark,
+         class T,
+         unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int BinSize = BlockSize,
+         unsigned int Trials  = 100>
 void run_benchmark(benchmark::State& state, hipStream_t stream, size_t N)
 {
     // Make sure size is a multiple of BlockSize
     constexpr auto items_per_block = BlockSize * ItemsPerThread;
-    const auto size = items_per_block * ((N + items_per_block - 1)/items_per_block);
-    const auto bin_size = BinSize * ((N + items_per_block - 1)/items_per_block);
+    const auto     size     = items_per_block * ((N + items_per_block - 1) / items_per_block);
+    const auto     bin_size = BinSize * ((N + items_per_block - 1) / items_per_block);
     // Allocate and fill memory
     std::vector<T> input(size, 0.0f);
-    T * d_input;
-    T * d_output;
+    T*             d_input;
+    T*             d_output;
     HIP_CHECK(hipMalloc(&d_input, size * sizeof(T)));
     HIP_CHECK(hipMalloc(&d_output, bin_size * sizeof(T)));
-    HIP_CHECK(
-        hipMemcpy(
-            d_input, input.data(),
-            size * sizeof(T),
-            hipMemcpyHostToDevice
-        )
-    );
+    HIP_CHECK(hipMemcpy(d_input, input.data(), size * sizeof(T), hipMemcpyHostToDevice));
     HIP_CHECK(hipDeviceSynchronize());
 
-    for (auto _ : state)
+    for(auto _ : state)
     {
         auto start = std::chrono::high_resolution_clock::now();
         hipLaunchKernelGGL(
             HIP_KERNEL_NAME(kernel<Benchmark, T, BlockSize, ItemsPerThread, BinSize, Trials>),
-            dim3(size/items_per_block), dim3(BlockSize), 0, stream,
-            d_input, d_output
-        );
+            dim3(size / items_per_block),
+            dim3(BlockSize),
+            0,
+            stream,
+            d_input,
+            d_output);
         HIP_CHECK(hipPeekAtLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
+        auto elapsed_seconds
+            = std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
 
         state.SetIterationTime(elapsed_seconds.count());
     }
@@ -143,41 +131,38 @@ void run_benchmark(benchmark::State& state, hipStream_t stream, size_t N)
 }
 
 // IPT - items per thread
-#define CREATE_BENCHMARK(T, BS, IPT) \
-    benchmark::RegisterBenchmark( \
-        (std::string("block_histogram<Datatype:"#T",Block Size:"#BS",Items Per Thread:"#IPT",SubAlgorithm Name:" + algorithm_name + ">.Method Name:") + method_name).c_str(), \
-        &run_benchmark<Benchmark, T, BS, IPT>, \
-        stream, size \
-    )
+#define CREATE_BENCHMARK(T, BS, IPT)                                                            \
+    benchmark::RegisterBenchmark(std::string("block_histogram<data_type:" #T ",block_size:" #BS \
+                                             ",items_per_thread:" #IPT ",sub_algorithm_name:"   \
+                                             + algorithm_name + ">.method_name:" + method_name) \
+                                     .c_str(),                                                  \
+                                 &run_benchmark<Benchmark, T, BS, IPT>,                         \
+                                 stream,                                                        \
+                                 size)
 
-#define BENCHMARK_TYPE(type, block) \
-    CREATE_BENCHMARK(type, block, 1), \
-    CREATE_BENCHMARK(type, block, 2), \
-    CREATE_BENCHMARK(type, block, 3), \
-    CREATE_BENCHMARK(type, block, 4), \
-    CREATE_BENCHMARK(type, block, 8), \
-    CREATE_BENCHMARK(type, block, 16)
+#define BENCHMARK_TYPE(type, block)                                         \
+    CREATE_BENCHMARK(type, block, 1), CREATE_BENCHMARK(type, block, 2),     \
+        CREATE_BENCHMARK(type, block, 3), CREATE_BENCHMARK(type, block, 4), \
+        CREATE_BENCHMARK(type, block, 8), CREATE_BENCHMARK(type, block, 16)
 
 template<class Benchmark>
 void add_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                    const std::string& method_name,
-                    const std::string& algorithm_name,
-                    hipStream_t stream,
-                    size_t size)
+                    const std::string&                            method_name,
+                    const std::string&                            algorithm_name,
+                    hipStream_t                                   stream,
+                    size_t                                        size)
 {
-    std::vector<benchmark::internal::Benchmark*> new_benchmarks =
-    {
-        BENCHMARK_TYPE(int, 256),
-        BENCHMARK_TYPE(int, 320),
-        BENCHMARK_TYPE(int, 512),
+    std::vector<benchmark::internal::Benchmark*> new_benchmarks
+        = {BENCHMARK_TYPE(int, 256),
+           BENCHMARK_TYPE(int, 320),
+           BENCHMARK_TYPE(int, 512),
 
-        BENCHMARK_TYPE(unsigned long long, 256),
-        BENCHMARK_TYPE(unsigned long long, 320)
-    };
+           BENCHMARK_TYPE(unsigned long long, 256),
+           BENCHMARK_TYPE(unsigned long long, 320)};
     benchmarks.insert(benchmarks.end(), new_benchmarks.begin(), new_benchmarks.end());
 }
 
-int main(int argc, char *argv[])
+int main(int argc, char* argv[])
 {
     cli::Parser parser(argc, argv);
     parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
@@ -186,15 +171,15 @@ int main(int argc, char *argv[])
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
-    const size_t size = parser.get<size_t>("size");
-    const int trials = parser.get<int>("trials");
+    const size_t size   = parser.get<size_t>("size");
+    const int    trials = parser.get<int>("trials");
 
     std::cout << "benchmark_block_histogram" << std::endl;
 
     // HIP
-    hipStream_t stream = 0; // default
+    hipStream_t     stream = 0; // default
     hipDeviceProp_t devProp;
-    int device_id = 0;
+    int             device_id = 0;
     HIP_CHECK(hipGetDevice(&device_id));
     HIP_CHECK(hipGetDeviceProperties(&devProp, device_id));
     std::cout << "[HIP] Device name: " << devProp.name << std::endl;
@@ -203,14 +188,10 @@ int main(int argc, char *argv[])
     std::vector<benchmark::internal::Benchmark*> benchmarks;
     // using_atomic
     using histogram_a_t = histogram<hipcub::BlockHistogramAlgorithm::BLOCK_HISTO_ATOMIC>;
-    add_benchmarks<histogram_a_t>(
-        benchmarks, "histogram", "using_atomic", stream, size
-    );
+    add_benchmarks<histogram_a_t>(benchmarks, "histogram", "using_atomic", stream, size);
     // using_sort
     using histogram_s_t = histogram<hipcub::BlockHistogramAlgorithm::BLOCK_HISTO_SORT>;
-    add_benchmarks<histogram_s_t>(
-        benchmarks, "histogram", "using_sort", stream, size
-    );
+    add_benchmarks<histogram_s_t>(benchmarks, "histogram", "using_sort", stream, size);
 
     // Use manual timing
     for(auto& b : benchmarks)
