@@ -9,8 +9,8 @@
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
 //
-// The above copyright notice and this permission notice shall be included in all
-// copies or substantial portions of the Software.
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -26,21 +26,16 @@
 #include "hipcub/block/block_reduce.hpp"
 #include "hipcub/thread/thread_operators.hpp"
 
-
 #ifndef DEFAULT_N
 const size_t DEFAULT_N = 1024 * 1024 * 32;
 #endif
 
-template<
-    class Runner,
-    class T,
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    unsigned int Trials
->
-__global__
-__launch_bounds__(BlockSize)
-void kernel(const T* input, T* output)
+template<class Runner,
+         class T,
+         unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int Trials>
+__global__ __launch_bounds__(BlockSize) void kernel(const T* input, T* output)
 {
     Runner::template run<T, BlockSize, ItemsPerThread, Trials>(input, output);
 }
@@ -48,14 +43,8 @@ void kernel(const T* input, T* output)
 template<hipcub::BlockReduceAlgorithm algorithm>
 struct reduce
 {
-    template<
-        class T,
-        unsigned int BlockSize,
-        unsigned int ItemsPerThread,
-        unsigned int Trials
-    >
-    __device__
-    static void run(const T* input, T* output)
+    template<class T, unsigned int BlockSize, unsigned int ItemsPerThread, unsigned int Trials>
+    __device__ static void run(const T* input, T* output)
     {
         const unsigned int i = hipBlockIdx_x * hipBlockDim_x + hipThreadIdx_x;
 
@@ -69,11 +58,11 @@ struct reduce
         using breduce_t = hipcub::BlockReduce<T, BlockSize, algorithm>;
         __shared__ typename breduce_t::TempStorage storage;
 
-        #pragma nounroll
+#pragma nounroll
         for(unsigned int trial = 0; trial < Trials; trial++)
         {
             reduced_value = breduce_t(storage).Reduce(values, hipcub::Sum());
-            values[0] = reduced_value;
+            values[0]     = reduced_value;
         }
 
         if(hipThreadIdx_x == 0)
@@ -83,47 +72,41 @@ struct reduce
     }
 };
 
-template<
-    class Benchmark,
-    class T,
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    unsigned int Trials = 100
->
+template<class Benchmark,
+         class T,
+         unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int Trials = 100>
 void run_benchmark(benchmark::State& state, hipStream_t stream, size_t N)
 {
     // Make sure size is a multiple of BlockSize
     constexpr auto items_per_block = BlockSize * ItemsPerThread;
-    const auto size = items_per_block * ((N + items_per_block - 1)/items_per_block);
+    const auto     size = items_per_block * ((N + items_per_block - 1) / items_per_block);
     // Allocate and fill memory
     std::vector<T> input(size, T(1));
-    T * d_input;
-    T * d_output;
+    T*             d_input;
+    T*             d_output;
     HIP_CHECK(hipMalloc(&d_input, size * sizeof(T)));
     HIP_CHECK(hipMalloc(&d_output, size * sizeof(T)));
-    HIP_CHECK(
-        hipMemcpy(
-            d_input, input.data(),
-            size * sizeof(T),
-            hipMemcpyHostToDevice
-        )
-    );
+    HIP_CHECK(hipMemcpy(d_input, input.data(), size * sizeof(T), hipMemcpyHostToDevice));
     HIP_CHECK(hipDeviceSynchronize());
 
-    for (auto _ : state)
+    for(auto _ : state)
     {
         auto start = std::chrono::high_resolution_clock::now();
-        hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(kernel<Benchmark, T, BlockSize, ItemsPerThread, Trials>),
-            dim3(size/items_per_block), dim3(BlockSize), 0, stream,
-            d_input, d_output
-        );
+        hipLaunchKernelGGL(HIP_KERNEL_NAME(kernel<Benchmark, T, BlockSize, ItemsPerThread, Trials>),
+                           dim3(size / items_per_block),
+                           dim3(BlockSize),
+                           0,
+                           stream,
+                           d_input,
+                           d_output);
         HIP_CHECK(hipPeekAtLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
+        auto elapsed_seconds
+            = std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
 
         state.SetIterationTime(elapsed_seconds.count());
     }
@@ -135,32 +118,30 @@ void run_benchmark(benchmark::State& state, hipStream_t stream, size_t N)
 }
 
 // IPT - items per thread
-#define CREATE_BENCHMARK(T, BS, IPT) \
-    benchmark::RegisterBenchmark( \
-        (std::string("block_reduce<Datatype:"#T",Block Size:"#BS",Items Per Thread:"#IPT",SubAlgorithm Name:" + algorithm_name + ">.Method Name:") + method_name).c_str(), \
-        &run_benchmark<Benchmark, T, BS, IPT>, \
-        stream, size \
-    )
+#define CREATE_BENCHMARK(T, BS, IPT)                                                            \
+    benchmark::RegisterBenchmark(std::string("block_reduce<data_type:" #T ",block_size:" #BS    \
+                                             ",items_per_thread:" #IPT ",sub_algorithm_name:"   \
+                                             + algorithm_name + ">.method_name:" + method_name) \
+                                     .c_str(),                                                  \
+                                 &run_benchmark<Benchmark, T, BS, IPT>,                         \
+                                 stream,                                                        \
+                                 size)
 
-#define BENCHMARK_TYPE(type, block) \
-    CREATE_BENCHMARK(type, block, 1), \
-    CREATE_BENCHMARK(type, block, 2), \
-    CREATE_BENCHMARK(type, block, 3), \
-    CREATE_BENCHMARK(type, block, 4), \
-    CREATE_BENCHMARK(type, block, 8), \
-    CREATE_BENCHMARK(type, block, 11), \
-    CREATE_BENCHMARK(type, block, 16)
+#define BENCHMARK_TYPE(type, block)                                          \
+    CREATE_BENCHMARK(type, block, 1), CREATE_BENCHMARK(type, block, 2),      \
+        CREATE_BENCHMARK(type, block, 3), CREATE_BENCHMARK(type, block, 4),  \
+        CREATE_BENCHMARK(type, block, 8), CREATE_BENCHMARK(type, block, 11), \
+        CREATE_BENCHMARK(type, block, 16)
 
 template<class Benchmark>
 void add_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                    const std::string& method_name,
-                    const std::string& algorithm_name,
-                    hipStream_t stream,
-                    size_t size)
+                    const std::string&                            method_name,
+                    const std::string&                            algorithm_name,
+                    hipStream_t                                   stream,
+                    size_t                                        size)
 {
 
-    std::vector<benchmark::internal::Benchmark*> new_benchmarks =
-    {
+    std::vector<benchmark::internal::Benchmark*> new_benchmarks = {
         // When block size is less than or equal to warp size
         BENCHMARK_TYPE(int, 64),
         BENCHMARK_TYPE(float, 64),
@@ -177,7 +158,7 @@ void add_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
     benchmarks.insert(benchmarks.end(), new_benchmarks.begin(), new_benchmarks.end());
 }
 
-int main(int argc, char *argv[])
+int main(int argc, char* argv[])
 {
     cli::Parser parser(argc, argv);
     parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
@@ -186,15 +167,15 @@ int main(int argc, char *argv[])
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
-    const size_t size = parser.get<size_t>("size");
-    const int trials = parser.get<int>("trials");
+    const size_t size   = parser.get<size_t>("size");
+    const int    trials = parser.get<int>("trials");
 
     std::cout << "benchmark_block_reduce" << std::endl;
 
     // HIP
-    hipStream_t stream = 0; // default
+    hipStream_t     stream = 0; // default
     hipDeviceProp_t devProp;
-    int device_id = 0;
+    int             device_id = 0;
     HIP_CHECK(hipGetDevice(&device_id));
     HIP_CHECK(hipGetDeviceProperties(&devProp, device_id));
     std::cout << "[HIP] Device name: " << devProp.name << std::endl;
@@ -203,19 +184,22 @@ int main(int argc, char *argv[])
     std::vector<benchmark::internal::Benchmark*> benchmarks;
     // using_warp_scan
     using reduce_uwr_t = reduce<hipcub::BlockReduceAlgorithm::BLOCK_REDUCE_WARP_REDUCTIONS>;
-    add_benchmarks<reduce_uwr_t>(
-        benchmarks, "reduce", "BLOCK_REDUCE_WARP_REDUCTIONS", stream, size
-    );
+    add_benchmarks<reduce_uwr_t>(benchmarks,
+                                 "reduce",
+                                 "BLOCK_REDUCE_WARP_REDUCTIONS",
+                                 stream,
+                                 size);
     // raking reduce
     using reduce_rr_t = reduce<hipcub::BlockReduceAlgorithm::BLOCK_REDUCE_RAKING>;
-    add_benchmarks<reduce_rr_t>(
-        benchmarks, "reduce", "BLOCK_REDUCE_RAKING", stream, size
-    );
+    add_benchmarks<reduce_rr_t>(benchmarks, "reduce", "BLOCK_REDUCE_RAKING", stream, size);
     // raking reduce commutative only
-    using reduce_rrco_t = reduce<hipcub::BlockReduceAlgorithm::BLOCK_REDUCE_RAKING_COMMUTATIVE_ONLY>;
-    add_benchmarks<reduce_rrco_t>(
-        benchmarks, "reduce", "BLOCK_REDUCE_RAKING_COMMUTATIVE_ONLY", stream, size
-    );
+    using reduce_rrco_t
+        = reduce<hipcub::BlockReduceAlgorithm::BLOCK_REDUCE_RAKING_COMMUTATIVE_ONLY>;
+    add_benchmarks<reduce_rrco_t>(benchmarks,
+                                  "reduce",
+                                  "BLOCK_REDUCE_RAKING_COMMUTATIVE_ONLY",
+                                  stream,
+                                  size);
 
     // Use manual timing
     for(auto& b : benchmarks)
