@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -9,8 +9,8 @@
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
 //
-// The above copyright notice and this permission notice shall be included in all
-// copies or substantial portions of the Software.
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -60,7 +60,8 @@ struct operation<no_operation, T, ItemsPerThread, BlockSize>
 {
     typedef empty_storage_type storage_type;
 
-    HIPCUB_DEVICE inline void operator()(storage_type& storage, T (&)[ItemsPerThread], T* = nullptr)
+    HIPCUB_DEVICE inline void
+        operator()(storage_type& /*storage*/, T (&)[ItemsPerThread], T* = nullptr) const
     {}
 };
 
@@ -72,7 +73,7 @@ struct operation<custom_operation, T, ItemsPerThread, BlockSize>
 
     HIPCUB_DEVICE inline void operator()(storage_type& storage,
                                          T (&input)[ItemsPerThread],
-                                         T* global_mem_output = nullptr)
+                                         T* global_mem_output = nullptr) const
     {
         (void)storage;
         (void)global_mem_output;
@@ -267,17 +268,11 @@ template<typename T,
 void run_benchmark(benchmark::State& state, size_t size, const hipStream_t stream)
 {
     const size_t   grid_size = size / (BlockSize * ItemsPerThread);
-    std::vector<T> input;
-    if(std::is_floating_point<T>::value)
-    {
-        input = benchmark_utils::get_random_data<T>(size, (T)-1000, (T) + 1000);
-    }
-    else
-    {
-        input = benchmark_utils::get_random_data<T>(size,
-                                                    std::numeric_limits<T>::min(),
-                                                    std::numeric_limits<T>::max());
-    }
+    std::vector<T> input
+        = benchmark_utils::get_random_data<T>(size,
+                                              benchmark_utils::generate_limits<T>::min(),
+                                              benchmark_utils::generate_limits<T>::max());
+
     T* d_input;
     T* d_output;
     HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&d_input), size * sizeof(T)));
@@ -348,21 +343,15 @@ void run_benchmark(benchmark::State& state, size_t size, const hipStream_t strea
 template<typename T>
 void run_benchmark_memcpy(benchmark::State& state, size_t size, const hipStream_t stream)
 {
-    std::vector<T> input;
-    if(std::is_floating_point<T>::value)
-    {
-        input = benchmark_utils::get_random_data<T>(size, (T)-1000, (T) + 1000);
-    }
-    else
-    {
-        input = benchmark_utils::get_random_data<T>(size,
-                                                    std::numeric_limits<T>::min(),
-                                                    std::numeric_limits<T>::max());
-    }
+    // Allocate device buffers
+    // Note: since this benchmark only tests memcpy performance between device
+    // buffers, we don't really need to copy data into these from the host -
+    // whatever happens to be in memory will suffice.
     T* d_input;
     T* d_output;
     HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&d_input), size * sizeof(T)));
     HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&d_output), size * sizeof(T)));
+
     // Warm-up
     for(size_t i = 0; i < 10; i++)
     {
@@ -406,20 +395,18 @@ void run_benchmark_memcpy(benchmark::State& state, size_t size, const hipStream_
     HIP_CHECK(hipFree(d_output));
 }
 
-#define CREATE_BENCHMARK_IPT(METHOD, OPERATION, T, SIZE, BLOCK_SIZE, IPT)                     \
-    {                                                                                         \
-        benchmarks.push_back(benchmark::RegisterBenchmark(                                    \
-            #METHOD "_" #OPERATION "<" #T "," #SIZE ",BS:" #BLOCK_SIZE ",IPT:" #IPT ">",      \
-            [=](benchmark::State& state)                                                      \
-            { run_benchmark<T, BLOCK_SIZE, IPT, METHOD, OPERATION>(state, SIZE, stream); })); \
-    }
+#define CREATE_BENCHMARK_IPT(METHOD, OPERATION, T, SIZE, BS, IPT)                             \
+    benchmarks.push_back(benchmark::RegisterBenchmark(                                        \
+        std::string("device_memory<method:" #METHOD ",operation:" #OPERATION ",data_type:" #T \
+                    ",size:" #SIZE ",block_size:" #BS ",items_per_thread:" #IPT ">.")         \
+            .c_str(),                                                                         \
+        [=](benchmark::State& state)                                                          \
+        { run_benchmark<T, BS, IPT, METHOD, OPERATION>(state, SIZE, stream); }));
 
-#define CREATE_BENCHMARK_MEMCPY(T, SIZE)                                                      \
-    {                                                                                         \
-        benchmarks.push_back(benchmark::RegisterBenchmark(                                    \
-            "Memcpy<" #T "," #SIZE ">",                                                       \
-            [=](benchmark::State& state) { run_benchmark_memcpy<T>(state, SIZE, stream); })); \
-    }
+#define CREATE_BENCHMARK_MEMCPY(T, SIZE)                                               \
+    benchmarks.push_back(benchmark::RegisterBenchmark(                                 \
+        std::string("device_memory_memcpy<data_type:" #T ",size:" #SIZE ">.").c_str(), \
+        [=](benchmark::State& state) { run_benchmark_memcpy<T>(state, SIZE, stream); }));
 
 // clang-format off
 #define CREATE_BENCHMARK_BLOCK_SIZE(MEM_OP, OP, TYPE, SIZE, BLOCK_SIZE) \
