@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2020 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -9,8 +9,8 @@
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
 //
-// The above copyright notice and this permission notice shall be included in all
-// copies or substantial portions of the Software.
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -25,61 +25,38 @@
 // HIP API
 #include "hipcub/device/device_reduce.hpp"
 
-
 #ifndef DEFAULT_N
 const size_t DEFAULT_N = 1024 * 1024 * 128;
 #endif
 
-const unsigned int batch_size = 10;
+const unsigned int batch_size  = 10;
 const unsigned int warmup_size = 5;
 
-template<
-    class T,
-    class OutputT,
-    class ReduceKernel
->
+template<class T, class OutputT, class ReduceKernel>
 void run_benchmark(benchmark::State& state,
-                   size_t size,
+                   size_t            size,
                    const hipStream_t stream,
-                   ReduceKernel reduce)
+                   ReduceKernel      reduce)
 {
     std::vector<T> input = benchmark_utils::get_random_data<T>(size, T(0), T(1000));
 
-    T * d_input;
-    OutputT * d_output;
+    T*       d_input;
+    OutputT* d_output;
     HIP_CHECK(hipMalloc(&d_input, size * sizeof(T)));
     HIP_CHECK(hipMalloc(&d_output, sizeof(OutputT)));
-    HIP_CHECK(
-        hipMemcpy(
-            d_input, input.data(),
-            size * sizeof(T),
-            hipMemcpyHostToDevice
-        )
-    );
+    HIP_CHECK(hipMemcpy(d_input, input.data(), size * sizeof(T), hipMemcpyHostToDevice));
     HIP_CHECK(hipDeviceSynchronize());
 
     // Allocate temporary storage memory
     size_t temp_storage_size_bytes = 0;
-    void * d_temp_storage = nullptr;
+    void*  d_temp_storage          = nullptr;
     // Get size of d_temp_storage
-    HIP_CHECK(
-        reduce(
-            d_temp_storage, temp_storage_size_bytes,
-            d_input, d_output, size,
-            stream, false
-        )
-    );
-    HIP_CHECK(hipMalloc(&d_temp_storage,temp_storage_size_bytes));
+    HIP_CHECK(reduce(d_temp_storage, temp_storage_size_bytes, d_input, d_output, size, stream));
+    HIP_CHECK(hipMalloc(&d_temp_storage, temp_storage_size_bytes));
     HIP_CHECK(hipDeviceSynchronize());
     for(size_t i = 0; i < warmup_size; i++)
     {
-        HIP_CHECK(
-            reduce(
-                d_temp_storage, temp_storage_size_bytes,
-                d_input, d_output, size,
-                stream, false
-            )
-        );
+        HIP_CHECK(reduce(d_temp_storage, temp_storage_size_bytes, d_input, d_output, size, stream));
     }
     HIP_CHECK(hipDeviceSynchronize());
 
@@ -90,18 +67,13 @@ void run_benchmark(benchmark::State& state,
         for(size_t i = 0; i < batch_size; i++)
         {
             HIP_CHECK(
-                reduce(
-                    d_temp_storage, temp_storage_size_bytes,
-                    d_input, d_output, size,
-                    stream, false
-                )
-            );
+                reduce(d_temp_storage, temp_storage_size_bytes, d_input, d_output, size, stream));
         }
         HIP_CHECK(hipStreamSynchronize(stream));
 
         auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
+        auto elapsed_seconds
+            = std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
         state.SetIterationTime(elapsed_seconds.count());
     }
     state.SetBytesProcessed(state.iterations() * batch_size * size * sizeof(T));
@@ -116,47 +88,56 @@ template<typename T, typename Op>
 struct Benchmark;
 
 template<typename T>
-struct Benchmark<T, hipcub::Sum> {
+struct Benchmark<T, hipcub::Sum>
+{
     static void run(benchmark::State& state, size_t size, const hipStream_t stream)
     {
-        run_benchmark<T, T>(state, size, stream, hipcub::DeviceReduce::Sum<T*, T*>);
+        hipError_t (*ptr_to_sum)(void*, size_t&, T*, T*, int, hipStream_t)
+            = &hipcub::DeviceReduce::Sum;
+        run_benchmark<T, T>(state, size, stream, ptr_to_sum);
     }
 };
 
 template<typename T>
-struct Benchmark<T, hipcub::Min> {
+struct Benchmark<T, hipcub::Min>
+{
     static void run(benchmark::State& state, size_t size, const hipStream_t stream)
     {
-        run_benchmark<T, T>(state, size, stream, hipcub::DeviceReduce::Min<T*, T*>);
+        hipError_t (*ptr_to_min)(void*, size_t&, T*, T*, int, hipStream_t)
+            = &hipcub::DeviceReduce::Min;
+        run_benchmark<T, T>(state, size, stream, ptr_to_min);
     }
 };
 
 template<typename T>
-struct Benchmark<T, hipcub::ArgMin> {
+struct Benchmark<T, hipcub::ArgMin>
+{
     using Difference = int;
-    using Iterator = typename hipcub::ArgIndexInputIterator<T*, Difference>;
-    using KeyValue = typename Iterator::value_type;
+    using Iterator   = typename hipcub::ArgIndexInputIterator<T*, Difference>;
+    using KeyValue   = typename Iterator::value_type;
 
     static void run(benchmark::State& state, size_t size, const hipStream_t stream)
     {
-        run_benchmark<T, KeyValue>(state, size, stream, hipcub::DeviceReduce::ArgMin<T*, KeyValue*>);
+        hipError_t (*ptr_to_argmin)(void*, size_t&, T*, KeyValue*, int, hipStream_t)
+            = &hipcub::DeviceReduce::ArgMin;
+        run_benchmark<T, KeyValue>(state, size, stream, ptr_to_argmin);
     }
 };
 
-#define CREATE_BENCHMARK(T, REDUCE_OP) \
-benchmark::RegisterBenchmark( \
-    ("reduce<Datatype:" #T ",Op:" #REDUCE_OP ">"), \
-    &Benchmark<T, REDUCE_OP>::run, size, stream \
-)
+#define CREATE_BENCHMARK(T, REDUCE_OP)                                                \
+    benchmark::RegisterBenchmark(std::string("device_reduce"                          \
+                                             "<data_type:" #T ",op:" #REDUCE_OP ">.") \
+                                     .c_str(),                                        \
+                                 &Benchmark<T, REDUCE_OP>::run,                       \
+                                 size,                                                \
+                                 stream)
 
-#define CREATE_BENCHMARKS(REDUCE_OP) \
-    CREATE_BENCHMARK(int, REDUCE_OP), \
-    CREATE_BENCHMARK(long long, REDUCE_OP), \
-    CREATE_BENCHMARK(float, REDUCE_OP), \
-    CREATE_BENCHMARK(double, REDUCE_OP), \
-    CREATE_BENCHMARK(int8_t, REDUCE_OP)
+#define CREATE_BENCHMARKS(REDUCE_OP)                                             \
+    CREATE_BENCHMARK(int, REDUCE_OP), CREATE_BENCHMARK(long long, REDUCE_OP),    \
+        CREATE_BENCHMARK(float, REDUCE_OP), CREATE_BENCHMARK(double, REDUCE_OP), \
+        CREATE_BENCHMARK(int8_t, REDUCE_OP)
 
-int main(int argc, char *argv[])
+int main(int argc, char* argv[])
 {
     cli::Parser parser(argc, argv);
     parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
@@ -165,15 +146,15 @@ int main(int argc, char *argv[])
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
-    const size_t size = parser.get<size_t>("size");
-    const int trials = parser.get<int>("trials");
+    const size_t size   = parser.get<size_t>("size");
+    const int    trials = parser.get<int>("trials");
 
     std::cout << "benchmark_device_reduce" << std::endl;
 
     // HIP
-    hipStream_t stream = 0; // default
+    hipStream_t     stream = 0; // default
     hipDeviceProp_t devProp;
-    int device_id = 0;
+    int             device_id = 0;
     HIP_CHECK(hipGetDevice(&device_id));
     HIP_CHECK(hipGetDeviceProperties(&devProp, device_id));
     std::cout << "[HIP] Device name: " << devProp.name << std::endl;
@@ -181,18 +162,17 @@ int main(int argc, char *argv[])
     using custom_double2 = benchmark_utils::custom_type<double, double>;
 
     // Add benchmarks
-    std::vector<benchmark::internal::Benchmark*> benchmarks =
-    {
+    std::vector<benchmark::internal::Benchmark*> benchmarks = {
         CREATE_BENCHMARKS(hipcub::Sum),
         CREATE_BENCHMARK(custom_double2, hipcub::Sum),
         CREATE_BENCHMARKS(hipcub::Min),
-        #ifdef HIPCUB_ROCPRIM_API
+#ifdef HIPCUB_ROCPRIM_API
         CREATE_BENCHMARK(custom_double2, hipcub::Min),
-        #endif
+#endif
         CREATE_BENCHMARKS(hipcub::ArgMin),
-        #ifdef HIPCUB_ROCPRIM_API
+#ifdef HIPCUB_ROCPRIM_API
         CREATE_BENCHMARK(custom_double2, hipcub::ArgMin),
-        #endif
+#endif
     };
 
     // Use manual timing
