@@ -1,6 +1,6 @@
 /******************************************************************************
 * Copyright (c) 2011-2021, NVIDIA CORPORATION.  All rights reserved.
-* Modifications Copyright (c) 2021, Advanced Micro Devices, Inc.  All rights reserved.
+* Modifications Copyright (c) 2021-2024, Advanced Micro Devices, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
@@ -29,6 +29,8 @@
 #ifndef HIPCUB_ROCPRIM_BLOCK_BLOCK_MERGE_SORT_HPP_
 #define HIPCUB_ROCPRIM_BLOCK_BLOCK_MERGE_SORT_HPP_
 
+#include "../../../config.hpp"
+
 #include "../thread/thread_sort.hpp"
 #include "../util_math.hpp"
 #include "../util_type.hpp"
@@ -203,7 +205,6 @@ private:
   const unsigned int linear_tid;
 
 public:
-  /// \smemstorage{BlockMergeSort}
   struct TempStorage : Uninitialized<_TempStorage> {};
 
   BlockMergeSortStrategy() = delete;
@@ -704,7 +705,6 @@ private:
  *   arithmetic types into ascending/descending order.
  *
  * @par A Simple Example
- * @blockcollective{BlockMergeSort}
  * @par
  * The code snippet below illustrates a sort of 512 integer keys that are
  * partitioned across 128 threads * where each thread owns 4 consecutive items.
