@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2017-2023, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2017-2024, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -30,13 +30,14 @@
 #ifndef HIPCUB_ROCPRIM_UTIL_PTX_HPP_
 #define HIPCUB_ROCPRIM_UTIL_PTX_HPP_
 
-#include <cstdint>
-#include <type_traits>
-
 #include "../../config.hpp"
+#include "util_type.hpp"
 
 #include <rocprim/intrinsics/warp_shuffle.hpp>
 
+#include <cstdint>
+#include <type_traits>
+
 BEGIN_HIPCUB_NAMESPACE
 
 // Missing compared to CUB:
@@ -242,6 +243,21 @@ unsigned int BFE(UnsignedBits source,
     return detail::unsigned_bit_extract(source, bit_start, num_bits);
 }
 
+#if HIPCUB_IS_INT128_ENABLED
+/**
+ * Bitfield-extract for 128-bit types.
+ */
+template<typename UnsignedBits>
+__device__ __forceinline__ unsigned int BFE(UnsignedBits source,
+                                            unsigned int bit_start,
+                                            unsigned int num_bits,
+                                            Int2Type<16> /*byte_len*/)
+{
+    const __uint128_t MASK = (__uint128_t{1} << num_bits) - 1;
+    return (source >> bit_start) & MASK;
+}
+#endif
+
 // Bitfield insert.
 // Inserts the \p num_bits least significant bits of \p y into \p x at bit-offset \p bit_start.
 HIPCUB_DEVICE inline
