@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2021, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2021-2024, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -30,16 +30,18 @@
 #ifndef HIPCUB_ROCPRIM_UTIL_TYPE_HPP_
 #define HIPCUB_ROCPRIM_UTIL_TYPE_HPP_
 
-#include <limits>
-#include <type_traits>
-
 #include "../../config.hpp"
 
 #include <rocprim/detail/various.hpp>
+#include <rocprim/thread/radix_key_codec.hpp>
 #include <rocprim/types/future_value.hpp>
 
-#include <hip/hip_fp16.h>
 #include <hip/hip_bfloat16.h>
+#include <hip/hip_fp16.h>
+#include <hip/hip_vector_types.h>
+
+#include <limits>
+#include <type_traits>
 
 BEGIN_HIPCUB_NAMESPACE
 
@@ -49,6 +51,22 @@ using NullType = ::rocprim::empty_type;
 
 #endif
 
+#ifndef HIPCUB_IS_INT128_ENABLED
+    #if defined(__HIPCC_RTC__)
+        #if defined(__HIPCC_RTC_INT128__)
+            #define HIPCUB_IS_INT128_ENABLED 1
+        #endif // !defined(__HIPCC_RTC_INT128__)
+    #else // !defined(__HIPCC_RTC__)
+        #if HIP_VERSION >= 50400000
+            #if(HIPCUB_HOST_COMPILER == HIPCUB_HOST_COMPILER_GCC)                         \
+                || (HIPCUB_HOST_COMPILER == HIPCUB_HOST_COMPILER_CLANG) || defined(__ICC) \
+                || defined(__GNUC__) || defined(__clang__)
+                #define HIPCUB_IS_INT128_ENABLED 1
+            #endif // GCC || CLANG || ICC
+        #endif // VER >= 5.4
+    #endif // !defined(__HIPCC_RTC__)
+#endif // !defined(HIPCUB_IS_INT128_ENABLED)
+
 template<bool B, typename T, typename F> struct
 [[deprecated("[Since 1.16] If is deprecated use std::conditional instead.")]] If
 {
@@ -345,11 +363,98 @@ template <typename T> struct UnitWord<volatile T> : UnitWord<T> {};
 template <typename T> struct UnitWord<const T> : UnitWord<T> {};
 template <typename T> struct UnitWord<const volatile T> : UnitWord<T> {};
 
-
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
+/******************************************************************************
+ * Vector type inference utilities.
+ ******************************************************************************/
 
+template<typename T, int vec_elements>
+struct GenericCubVector
+{
+    static_assert(!sizeof(T), "CubVector can only have 1-4 elements");
+};
 
+enum
+{
+    /// The maximum number of elements in HIP vector types
+    MAX_VEC_ELEMENTS = 4,
+};
+
+template<typename T>
+struct GenericCubVector<T, 1>
+{
+    T x;
+
+    using BaseType = T;
+    using Type     = GenericCubVector<T, 1>;
+};
+
+template<typename T>
+struct GenericCubVector<T, 2>
+{
+    T x;
+    T y;
+
+    using BaseType = T;
+    using Type     = GenericCubVector<T, 2>;
+};
+
+template<typename T>
+struct GenericCubVector<T, 3>
+{
+    T x;
+    T y;
+    T z;
+
+    using BaseType = T;
+    using Type     = GenericCubVector<T, 3>;
+};
+
+template<typename T>
+struct GenericCubVector<T, 4>
+{
+    T x;
+    T y;
+    T z;
+    T w;
+
+    using BaseType = T;
+    using Type     = GenericCubVector<T, 4>;
+};
+
+template<typename T, int vec_elements>
+struct CubVectorType
+{
+    // Fallback on GenericCubVector
+    using Type = GenericCubVector<T, vec_elements>;
+};
+
+template<typename T, int vec_elements>
+using CubVector = typename CubVectorType<T, vec_elements>::Type;
+
+#define HIPCUB_DEFINE_VECTOR_TYPE(base_type, vec_type)        \
+    template<int vec_elements>                                \
+    struct CubVectorType<base_type, vec_elements>             \
+    {                                                         \
+        using Type = HIP_vector_type<vec_type, vec_elements>; \
+    };
+
+HIPCUB_DEFINE_VECTOR_TYPE(char, char)
+HIPCUB_DEFINE_VECTOR_TYPE(unsigned char, unsigned char)
+HIPCUB_DEFINE_VECTOR_TYPE(short, short)
+HIPCUB_DEFINE_VECTOR_TYPE(ushort, ushort)
+HIPCUB_DEFINE_VECTOR_TYPE(int, int)
+HIPCUB_DEFINE_VECTOR_TYPE(uint, uint)
+HIPCUB_DEFINE_VECTOR_TYPE(long, long)
+HIPCUB_DEFINE_VECTOR_TYPE(unsigned long, unsigned long)
+HIPCUB_DEFINE_VECTOR_TYPE(long long, long long)
+HIPCUB_DEFINE_VECTOR_TYPE(unsigned long long, unsigned long long)
+HIPCUB_DEFINE_VECTOR_TYPE(float, float)
+HIPCUB_DEFINE_VECTOR_TYPE(double, double)
+HIPCUB_DEFINE_VECTOR_TYPE(bool, unsigned char)
+
+#undef HIPCUB_DEFINE_VECTOR_TYPE
 
 /******************************************************************************
  * Wrapper types
@@ -390,7 +495,7 @@ struct Uninitialized
  *
  ******************************************************************************/
 
- #ifndef DOXYGEN_SHOULD_SKIP_THIS    // Do not document
+#ifndef DOXYGEN_SHOULD_SKIP_THIS // Do not document
 
 /**
  * \brief Basic type traits categories
@@ -438,6 +543,7 @@ struct BaseTraits<UNSIGNED_INTEGER, true, false, _UnsignedBits, T>
         NULL_TYPE       = false,
     };
 
+    using key_codec = rocprim::radix_key_codec<T>;
 
     static HIPCUB_HOST_DEVICE __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
     {
@@ -486,6 +592,8 @@ struct BaseTraits<SIGNED_INTEGER, true, false, _UnsignedBits, T>
         NULL_TYPE       = false,
     };
 
+    using key_codec = rocprim::radix_key_codec<T>;
+
     static HIPCUB_HOST_DEVICE __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
     {
         return key ^ HIGH_BIT;
@@ -577,6 +685,8 @@ struct BaseTraits<FLOATING_POINT, true, false, _UnsignedBits, T>
     static const UnsignedBits   LOWEST_KEY  = UnsignedBits(-1);
     static const UnsignedBits   MAX_KEY     = UnsignedBits(-1) ^ HIGH_BIT;
 
+    using key_codec = rocprim::radix_key_codec<T>;
+
     enum
     {
         PRIMITIVE       = true,
@@ -625,6 +735,83 @@ template <> struct NumericTraits<unsigned int> :        BaseTraits<UNSIGNED_INTE
 template <> struct NumericTraits<unsigned long> :       BaseTraits<UNSIGNED_INTEGER, true, false, unsigned long, unsigned long> {};
 template <> struct NumericTraits<unsigned long long> :  BaseTraits<UNSIGNED_INTEGER, true, false, unsigned long long, unsigned long long> {};
 
+    #if HIPCUB_IS_INT128_ENABLED
+template<>
+struct NumericTraits<__uint128_t>
+{
+    using T            = __uint128_t;
+    using UnsignedBits = __uint128_t;
+
+    static constexpr Category     CATEGORY   = UNSIGNED_INTEGER;
+    static constexpr UnsignedBits LOWEST_KEY = UnsignedBits(0);
+    static constexpr UnsignedBits MAX_KEY    = UnsignedBits(-1);
+
+    static constexpr bool PRIMITIVE = false;
+    static constexpr bool NULL_TYPE = false;
+
+    using key_codec = rocprim::radix_key_codec<T>;
+
+    static __host__ __device__ __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
+    {
+        return key;
+    }
+
+    static __host__ __device__ __forceinline__ UnsignedBits TwiddleOut(UnsignedBits key)
+    {
+        return key;
+    }
+
+    static __host__ __device__ __forceinline__ T Max()
+    {
+        return MAX_KEY;
+    }
+
+    static __host__ __device__ __forceinline__ T Lowest()
+    {
+        return LOWEST_KEY;
+    }
+};
+
+template<>
+struct NumericTraits<__int128_t>
+{
+    using T            = __int128_t;
+    using UnsignedBits = __uint128_t;
+
+    static constexpr Category     CATEGORY   = SIGNED_INTEGER;
+    static constexpr UnsignedBits HIGH_BIT   = UnsignedBits(1) << ((sizeof(UnsignedBits) * 8) - 1);
+    static constexpr UnsignedBits LOWEST_KEY = HIGH_BIT;
+    static constexpr UnsignedBits MAX_KEY    = UnsignedBits(-1) ^ HIGH_BIT;
+
+    static constexpr bool PRIMITIVE = false;
+    static constexpr bool NULL_TYPE = false;
+
+    using key_codec = rocprim::radix_key_codec<T>;
+
+    static __host__ __device__ __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
+    {
+        return key ^ HIGH_BIT;
+    };
+
+    static __host__ __device__ __forceinline__ UnsignedBits TwiddleOut(UnsignedBits key)
+    {
+        return key ^ HIGH_BIT;
+    };
+
+    static __host__ __device__ __forceinline__ T Max()
+    {
+        UnsignedBits retval = MAX_KEY;
+        return reinterpret_cast<T&>(retval);
+    }
+
+    static __host__ __device__ __forceinline__ T Lowest()
+    {
+        UnsignedBits retval = LOWEST_KEY;
+        return reinterpret_cast<T&>(retval);
+    }
+};
+    #endif
+
 template <> struct NumericTraits<float> :               BaseTraits<FLOATING_POINT, true, false, unsigned int, float> {};
 template <> struct NumericTraits<double> :              BaseTraits<FLOATING_POINT, true, false, unsigned long long, double> {};
 template <> struct NumericTraits<__half> :              BaseTraits<FLOATING_POINT, true, false, unsigned short, __half> {};
