@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -141,6 +141,11 @@ TYPED_TEST(HipcubBlockReduceSingleValueTests, Reduce)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     constexpr auto algorithm = TestFixture::algorithm;
     constexpr size_t block_size = TestFixture::block_size;
 
@@ -167,13 +172,13 @@ TYPED_TEST(HipcubBlockReduceSingleValueTests, Reduce)
                                            test_utils::convert_to_device<T>(0));
         for(size_t i = 0; i < output.size() / block_size; i++)
         {
-            T value = 0;
+            acc_type value(0);
             for(size_t j = 0; j < block_size; j++)
             {
                 auto idx = i * block_size + j;
-                value += output[idx];
+                value    = binary_op_host(value, output[idx]);
             }
-            expected_reductions[i] = value;
+            expected_reductions[i] = static_cast<T>(value);
         }
 
         // Preparing device
@@ -207,10 +212,9 @@ TYPED_TEST(HipcubBlockReduceSingleValueTests, Reduce)
         );
 
         // Verifying results
-        for(size_t i = 0; i < output_reductions.size(); i++)
-        {
-            ASSERT_EQ(output_reductions[i], expected_reductions[i]);
-        }
+        test_utils::assert_near(output_reductions,
+                                expected_reductions,
+                                test_utils::precision<T>::value * block_size);
 
         HIP_CHECK(hipFree(device_output));
         HIP_CHECK(hipFree(device_output_reductions));
@@ -246,6 +250,11 @@ TYPED_TEST(HipcubBlockReduceSingleValueTests, ReduceValid)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     constexpr auto algorithm = TestFixture::algorithm;
 
     constexpr size_t block_size = TestFixture::block_size;
@@ -283,13 +292,13 @@ TYPED_TEST(HipcubBlockReduceSingleValueTests, ReduceValid)
                                            test_utils::convert_to_device<T>(0));
         for(size_t i = 0; i < output.size() / block_size; i++)
         {
-            T value = 0;
+            acc_type value(0);
             for(size_t j = 0; j < valid_items; j++)
             {
                 auto idx = i * block_size + j;
-                value += output[idx];
+                value    = binary_op_host(output[idx], value);
             }
-            expected_reductions[i] = value;
+            expected_reductions[i] = static_cast<T>(value);
         }
 
         // Preparing device
@@ -323,10 +332,9 @@ TYPED_TEST(HipcubBlockReduceSingleValueTests, ReduceValid)
         );
 
         // Verifying results
-        for(size_t i = 0; i < output_reductions.size(); i++)
-        {
-            ASSERT_EQ(output_reductions[i], expected_reductions[i]);
-        }
+        test_utils::assert_near(output_reductions,
+                                expected_reductions,
+                                test_utils::precision<T>::value * block_size);
 
         HIP_CHECK(hipFree(device_output));
         HIP_CHECK(hipFree(device_output_reductions));
@@ -415,7 +423,12 @@ TYPED_TEST(HipcubBlockReduceInputArrayTests, Reduce)
     SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    using T                           = typename TestFixture::type;
+    using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     constexpr auto algorithm = TestFixture::algorithm;
     constexpr size_t block_size = TestFixture::block_size;
     constexpr size_t items_per_thread = TestFixture::items_per_thread;
@@ -450,13 +463,13 @@ TYPED_TEST(HipcubBlockReduceInputArrayTests, Reduce)
                                            test_utils::convert_to_device<T>(0));
         for(size_t i = 0; i < output.size() / items_per_block; i++)
         {
-            test_utils::convert_to_native_t<T> value = 0;
+            acc_type value(0);
             for(size_t j = 0; j < items_per_block; j++)
             {
                 auto idx = i * items_per_block + j;
-                value += output[idx];
+                value    = binary_op_host(static_cast<acc_type>(output[idx]), value);
             }
-            expected_reductions[i] = test_utils::convert_to_device<T>(value);
+            expected_reductions[i] = static_cast<T>(value);
         }
 
         // Preparing device
@@ -498,13 +511,9 @@ TYPED_TEST(HipcubBlockReduceInputArrayTests, Reduce)
         );
 
         // Verifying results
-        for(size_t i = 0; i < output_reductions.size(); i++)
-        {
-            ASSERT_NEAR(test_utils::convert_to_native(output_reductions[i]),
-                        test_utils::convert_to_native(expected_reductions[i]),
-                        test_utils::convert_to_native(0.05)
-                            * test_utils::convert_to_native(expected_reductions[i]));
-        }
+        test_utils::assert_near(output_reductions,
+                                expected_reductions,
+                                test_utils::precision<T>::value * items_per_block);
 
         HIP_CHECK(hipFree(device_output));
         HIP_CHECK(hipFree(device_output_reductions));
