@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -25,72 +25,77 @@
 // hipcub API
 #include "hipcub/device/device_adjacent_difference.hpp"
 #include "hipcub/iterator/counting_input_iterator.hpp"
+#include "hipcub/iterator/discard_output_iterator.hpp"
 #include "hipcub/iterator/transform_input_iterator.hpp"
+
+#include "test_utils.hpp"
 #include "test_utils_data_generation.hpp"
 
-template <class InputIteratorT, class OutputIteratorT, class... Args>
+template<class InputIteratorT, class OutputIteratorT, class... Args>
 hipError_t dispatch_adjacent_difference(std::true_type /*left*/,
                                         std::true_type /*copy*/,
-                                        void* d_temp_storage,
-                                        size_t& temp_storage_bytes,
-                                        InputIteratorT d_input,
+                                        void*           d_temp_storage,
+                                        size_t&         temp_storage_bytes,
+                                        InputIteratorT  d_input,
                                         OutputIteratorT d_output,
                                         Args&&... args)
 {
-    return ::hipcub::DeviceAdjacentDifference::SubtractLeftCopy(
-        d_temp_storage, temp_storage_bytes, d_input, d_output,
-        std::forward<Args>(args)...
-    );
+    return ::hipcub::DeviceAdjacentDifference::SubtractLeftCopy(d_temp_storage,
+                                                                temp_storage_bytes,
+                                                                d_input,
+                                                                d_output,
+                                                                std::forward<Args>(args)...);
 }
 
-template <class InputIteratorT, class OutputIteratorT, class... Args>
+template<class InputIteratorT, class OutputIteratorT, class... Args>
 hipError_t dispatch_adjacent_difference(std::true_type /*left*/,
                                         std::false_type /*copy*/,
-                                        void* d_temp_storage,
-                                        size_t& temp_storage_bytes,
+                                        void*          d_temp_storage,
+                                        size_t&        temp_storage_bytes,
                                         InputIteratorT d_input,
                                         OutputIteratorT /*d_output*/,
                                         Args&&... args)
 {
-    return ::hipcub::DeviceAdjacentDifference::SubtractLeft(
-        d_temp_storage, temp_storage_bytes, d_input,
-        std::forward<Args>(args)...
-    );
+    return ::hipcub::DeviceAdjacentDifference::SubtractLeft(d_temp_storage,
+                                                            temp_storage_bytes,
+                                                            d_input,
+                                                            std::forward<Args>(args)...);
 }
 
-template <class InputIteratorT, class OutputIteratorT, class... Args>
+template<class InputIteratorT, class OutputIteratorT, class... Args>
 hipError_t dispatch_adjacent_difference(std::false_type /*left*/,
                                         std::true_type /*copy*/,
-                                        void* d_temp_storage,
-                                        size_t& temp_storage_bytes,
-                                        InputIteratorT d_input,
+                                        void*           d_temp_storage,
+                                        size_t&         temp_storage_bytes,
+                                        InputIteratorT  d_input,
                                         OutputIteratorT d_output,
                                         Args&&... args)
 {
-    return ::hipcub::DeviceAdjacentDifference::SubtractRightCopy(
-        d_temp_storage, temp_storage_bytes, d_input, d_output,
-        std::forward<Args>(args)...
-    );
+    return ::hipcub::DeviceAdjacentDifference::SubtractRightCopy(d_temp_storage,
+                                                                 temp_storage_bytes,
+                                                                 d_input,
+                                                                 d_output,
+                                                                 std::forward<Args>(args)...);
 }
 
-template <class InputIteratorT, class OutputIteratorT, class... Args>
+template<class InputIteratorT, class OutputIteratorT, class... Args>
 hipError_t dispatch_adjacent_difference(std::false_type /*left*/,
                                         std::false_type /*copy*/,
-                                        void* d_temp_storage,
-                                        size_t& temp_storage_bytes,
+                                        void*          d_temp_storage,
+                                        size_t&        temp_storage_bytes,
                                         InputIteratorT d_input,
                                         OutputIteratorT /*d_output*/,
                                         Args&&... args)
 {
-    return ::hipcub::DeviceAdjacentDifference::SubtractRight(
-        d_temp_storage, temp_storage_bytes, d_input,
-        std::forward<Args>(args)...
-    );
+    return ::hipcub::DeviceAdjacentDifference::SubtractRight(d_temp_storage,
+                                                             temp_storage_bytes,
+                                                             d_input,
+                                                             std::forward<Args>(args)...);
 }
 
-template <typename Output, typename T, typename BinaryFunction>
+template<typename Output, typename T, typename BinaryFunction>
 auto get_expected_result(const std::vector<T>& input,
-                         const BinaryFunction op,
+                         const BinaryFunction  op,
                          std::true_type /*left*/)
 {
     std::vector<Output> result(input.size());
@@ -98,9 +103,9 @@ auto get_expected_result(const std::vector<T>& input,
     return result;
 }
 
-template <typename Output, typename T, typename BinaryFunction>
+template<typename Output, typename T, typename BinaryFunction>
 auto get_expected_result(const std::vector<T>& input,
-                         const BinaryFunction op,
+                         const BinaryFunction  op,
                          std::false_type /*left*/)
 {
     std::vector<Output> result(input.size());
@@ -109,22 +114,23 @@ auto get_expected_result(const std::vector<T>& input,
     return result;
 }
 
-template<
-    class InputT,
-    class OutputT = InputT,
-    bool Left = true,
-    bool Copy = true
->
+template<class InputT,
+         class OutputT  = InputT,
+         bool Left      = true,
+         bool Copy      = true,
+         bool UseGraphs = false>
 struct params
 {
-    using input_type = InputT;
-    using output_type = OutputT;
-    static constexpr bool left = Left;
-    static constexpr bool copy = Copy;
+    using input_type                 = InputT;
+    using output_type                = OutputT;
+    static constexpr bool left       = Left;
+    static constexpr bool copy       = Copy;
+    static constexpr bool use_graphs = UseGraphs;
 };
 
 template<class Params>
-class HipcubDeviceAdjacentDifference : public ::testing::Test {
+class HipcubDeviceAdjacentDifference : public ::testing::Test
+{
 public:
     using params = Params;
 };
@@ -133,42 +139,12 @@ typedef ::testing::Types<params<int>,
                          params<int, double>,
                          params<int8_t, int8_t, true, false>,
                          params<float, float, false, true>,
-                         params<double, double, true, true>
-#ifndef __HIP_PLATFORM_NVIDIA__
-                         ,
-                         // Kernel doesn't work on NVidia.
+                         params<double, double, true, true>,
                          params<test_utils::half, test_utils::half>,
-                         params<test_utils::bfloat16, test_utils::bfloat16>
-#endif
-                         >
+                         params<test_utils::bfloat16, test_utils::bfloat16>,
+                         params<int, int, true, true, true>>
     Params;
 
-std::vector<size_t> get_sizes()
-{
-    std::vector<size_t> sizes = { 1, 10, 53, 211, 1024, 2345, 4096, 34567, (1 << 16) - 1220, (1 << 23) - 76543 };
-    const std::vector<size_t> random_sizes = test_utils::get_random_data<size_t>(10, 1, 100000, rand());
-    sizes.insert(sizes.end(), random_sizes.begin(), random_sizes.end());
-    return sizes;
-}
-
-std::vector<size_t> get_large_sizes(int seed_value)
-{
-    // clang-format off
-    std::vector<size_t> sizes = {
-        (size_t{1} << 32) - 1, size_t{1} << 32,
-        (size_t{1} << 35) - 1, size_t{1} << 35
-    };
-    // clang-format on
-    const std::vector<size_t> random_sizes
-        = test_utils::get_random_data<size_t>(2,
-                                              (size_t{1} << 30) + 1,
-                                              (size_t{1} << 35) - 2,
-                                              seed_value);
-    sizes.insert(sizes.end(), random_sizes.begin(), random_sizes.end());
-    std::sort(sizes.begin(), sizes.end());
-    return sizes;
-}
-
 TYPED_TEST_SUITE(HipcubDeviceAdjacentDifference, Params);
 
 TYPED_TEST(HipcubDeviceAdjacentDifference, SubtractLeftCopy)
@@ -180,19 +156,26 @@ TYPED_TEST(HipcubDeviceAdjacentDifference, SubtractLeftCopy)
     using input_type = typename TestFixture::params::input_type;
     static constexpr std::integral_constant<bool, TestFixture::params::left> left_constant{};
     static constexpr std::integral_constant<bool, TestFixture::params::copy> copy_constant{};
-    using output_type = std::conditional_t<copy_constant, input_type, typename TestFixture::params::output_type>;
-    static constexpr hipStream_t stream = 0;
-    static constexpr bool debug_synchronous = false;
+    using output_type
+        = std::conditional_t<copy_constant, input_type, typename TestFixture::params::output_type>;
     static constexpr ::hipcub::Difference op;
 
-    const auto sizes = get_sizes();
-    for (size_t size : sizes)
+    hipStream_t stream = 0;
+    if(TestFixture::params::use_graphs)
     {
-        SCOPED_TRACE(testing::Message() << "with size = " << size);
-        for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        for(size_t size : test_utils::get_sizes(seed_value))
         {
-            const unsigned int seed_value = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
 
             const auto input = test_utils::get_random_data<input_type>(
                 size,
@@ -200,66 +183,86 @@ TYPED_TEST(HipcubDeviceAdjacentDifference, SubtractLeftCopy)
                 test_utils::convert_to_device<input_type>(50),
                 seed_value);
 
-            input_type * d_input{};
-            output_type * d_output{};
+            input_type*  d_input{};
+            output_type* d_output{};
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_input, size * sizeof(d_input[0])));
-            if (copy_constant)
+            if(copy_constant)
             {
-                HIP_CHECK(test_common_utils::hipMallocHelper(&d_output, size * sizeof(d_output[0])));
+                HIP_CHECK(
+                    test_common_utils::hipMallocHelper(&d_output, size * sizeof(d_output[0])));
             }
             HIP_CHECK(
-                hipMemcpy(
-                    d_input, input.data(),
-                    size * sizeof(input_type),
-                    hipMemcpyHostToDevice
-                )
-            );
+                hipMemcpy(d_input, input.data(), size * sizeof(input_type), hipMemcpyHostToDevice));
 
             const auto expected = get_expected_result<output_type>(input, op, left_constant);
 
             size_t temporary_storage_bytes = 0;
-            HIP_CHECK(
-                dispatch_adjacent_difference(
-                    left_constant, copy_constant,
-                    nullptr, temporary_storage_bytes,
-                    d_input, d_output, size, op, stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(dispatch_adjacent_difference(left_constant,
+                                                   copy_constant,
+                                                   nullptr,
+                                                   temporary_storage_bytes,
+                                                   d_input,
+                                                   d_output,
+                                                   size,
+                                                   op,
+                                                   stream));
 
 #ifdef __HIP_PLATFORM_AMD__
             ASSERT_GT(temporary_storage_bytes, 0U);
 #endif
 
-            void * d_temporary_storage;
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
-
+            void* d_temporary_storage;
             HIP_CHECK(
-                dispatch_adjacent_difference(
-                    left_constant, copy_constant,
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_input, d_output, size, op, stream, debug_synchronous
-                )
-            );
+                test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
+
+            hipGraph_t graph;
+            if(TestFixture::params::use_graphs)
+            {
+                graph = test_utils::createGraphHelper(stream);
+            }
+
+            HIP_CHECK(dispatch_adjacent_difference(left_constant,
+                                                   copy_constant,
+                                                   d_temporary_storage,
+                                                   temporary_storage_bytes,
+                                                   d_input,
+                                                   d_output,
+                                                   size,
+                                                   op,
+                                                   stream));
+
+            hipGraphExec_t graph_instance;
+            if(TestFixture::params::use_graphs)
+            {
+                graph_instance = test_utils::endCaptureGraphHelper(graph, stream, true, true);
+            }
 
             std::vector<output_type> output(size);
-            HIP_CHECK(
-                hipMemcpy(
-                    output.data(), copy_constant ? d_output : d_input,
-                    size * sizeof(output[0]),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            HIP_CHECK(hipMemcpy(output.data(),
+                                copy_constant ? d_output : d_input,
+                                size * sizeof(output[0]),
+                                hipMemcpyDeviceToHost));
 
             HIP_CHECK(hipFree(d_temporary_storage));
             HIP_CHECK(hipFree(d_input));
-            if (copy_constant)
+            if(copy_constant)
             {
                 HIP_CHECK(hipFree(d_output));
             }
 
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_bit_eq(output, expected));
+
+            if(TestFixture::params::use_graphs)
+            {
+                test_utils::cleanupGraphHelper(graph, graph_instance);
+            }
         }
     }
+
+    if(TestFixture::params::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
 
 // Params for tests
@@ -274,9 +277,8 @@ template<class Params>
 class HipcubDeviceAdjacentDifferenceLargeTests : public ::testing::Test
 {
 public:
-    static constexpr bool left              = Params::left;
-    static constexpr bool copy              = Params::copy;
-    static constexpr bool debug_synchronous = false;
+    static constexpr bool left = Params::left;
+    static constexpr bool copy = Params::copy;
 };
 
 using HipcubDeviceAdjacentDifferenceLargeTestsParams
@@ -297,85 +299,33 @@ struct discard_write
     {
         return value;
     }
-    __device__ discard_write& operator=(T)
+    __device__
+    discard_write&
+        operator=(T)
     {
         return *this;
     }
 };
 
-class discard_iterator
-{
-public:
-    struct discard_value
-    {
-        inline discard_value() = default;
-
-        template<class T>
-        HIPCUB_HOST_DEVICE inline discard_value(T){};
-
-        inline ~discard_value() = default;
-
-        template<class T>
-        HIPCUB_HOST_DEVICE inline discard_value& operator=(const T&)
-        {
-            return *this;
-        }
-    };
-
-    typedef discard_iterator self_type; ///< My own type
-    typedef discard_value    value_type; ///< The type of the element the iterator can point to
-    typedef discard_value*
-        pointer; ///< The type of a pointer to an element the iterator can point to
-    typedef discard_value
-        reference; ///< The type of a reference to an element the iterator can point to
-    typedef ptrdiff_t                       difference_type;
-    typedef std::random_access_iterator_tag iterator_category; ///< The iterator category
-    /// \brief Creates a new discard_iterator.
-    ///
-    /// \param index - optional index of discard iterator (default = 0).
-    HIPCUB_HOST_DEVICE inline discard_iterator(size_t index = 0) : index_(index) {}
-
-    inline ~discard_iterator() = default;
-
-    HIPCUB_HOST_DEVICE inline discard_value operator*() const
-    {
-        return discard_value();
-    }
-
-    HIPCUB_HOST_DEVICE inline discard_value operator[](difference_type distance) const
-    {
-        discard_iterator i = (*this) + distance;
-        return *i;
-    }
-
-    HIPCUB_HOST_DEVICE inline discard_iterator operator+(difference_type distance) const
-    {
-        auto i = static_cast<size_t>(static_cast<difference_type>(index_) + distance);
-        return discard_iterator(i);
-    }
-
-private:
-    mutable size_t index_;
-};
-
 template<class T, class InputIterator, class UnaryFunction>
-HIPCUB_HOST_DEVICE inline auto make_transform_iterator(InputIterator iterator,
-                                                       UnaryFunction transform)
+HIPCUB_HOST_DEVICE
+inline auto make_transform_iterator(InputIterator iterator, UnaryFunction transform)
 {
     return ::hipcub::TransformInputIterator<T, UnaryFunction, InputIterator>(iterator, transform);
 }
 
 template<class T>
-struct conversion_op : public std::unary_function<T, discard_write<T>>
+struct conversion_op
 {
-    HIPCUB_HOST_DEVICE auto operator()(const T i) const
+    HIPCUB_HOST_DEVICE
+    auto operator()(const T i) const
     {
         return discard_write<T>{i};
     }
 };
 
 template<class T>
-struct flag_expected_op : public std::binary_function<T, T, discard_write<T>>
+struct flag_expected_op
 {
     bool left;
     T    expected;
@@ -388,8 +338,8 @@ struct flag_expected_op : public std::binary_function<T, T, discard_write<T>>
         , d_flags(d_flags)
     {}
 
-    HIPCUB_HOST_DEVICE T operator()(const discard_write<T>& minuend,
-                                    const discard_write<T>& subtrahend)
+    HIPCUB_HOST_DEVICE
+    T operator()(const discard_write<T>& minuend, const discard_write<T>& subtrahend)
     {
         if(left)
         {
@@ -423,11 +373,10 @@ TYPED_TEST(HipcubDeviceAdjacentDifferenceLargeTests, LargeIndicesAndOpOnce)
     SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    using T                                 = size_t;
-    using OutputIterator                    = discard_iterator;
-    static constexpr bool left              = TestFixture::left;
-    static constexpr bool copy              = TestFixture::copy;
-    const bool            debug_synchronous = TestFixture::debug_synchronous;
+    using T                    = size_t;
+    using OutputIterator       = hipcub::DiscardOutputIterator<>;
+    static constexpr bool left = TestFixture::left;
+    static constexpr bool copy = TestFixture::copy;
 
     SCOPED_TRACE(testing::Message() << "left = " << left << ", copy = " << copy);
 
@@ -439,7 +388,7 @@ TYPED_TEST(HipcubDeviceAdjacentDifferenceLargeTests, LargeIndicesAndOpOnce)
             = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
 
-        const std::vector<size_t> sizes = get_large_sizes(seed_value);
+        const std::vector<size_t> sizes = test_utils::get_large_sizes(seed_value);
 
         for(const auto size : sizes)
         {
@@ -485,8 +434,7 @@ TYPED_TEST(HipcubDeviceAdjacentDifferenceLargeTests, LargeIndicesAndOpOnce)
                                                    output,
                                                    size,
                                                    flag_expected,
-                                                   stream,
-                                                   debug_synchronous));
+                                                   stream));
 
 #ifdef __HIP_PLATFORM_AMD__
             ASSERT_GT(temp_storage_size, 0U);
@@ -502,16 +450,15 @@ TYPED_TEST(HipcubDeviceAdjacentDifferenceLargeTests, LargeIndicesAndOpOnce)
                                                    output,
                                                    size,
                                                    flag_expected,
-                                                   stream,
-                                                   debug_synchronous));
+                                                   stream));
 
             // Copy output to host
             HIP_CHECK(hipMemcpy(flags, d_flags, sizeof(flags), hipMemcpyDeviceToHost));
 
             ASSERT_EQ(flags[0], 1);
             ASSERT_EQ(flags[1], 1);
-            hipFree(d_temp_storage);
-            hipFree(d_flags);
+            HIP_CHECK(hipFree(d_temp_storage));
+            HIP_CHECK(hipFree(d_flags));
         }
     }
 }
\ No newline at end of file
