@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -25,22 +25,24 @@
 // hipcub API
 #include "hipcub/device/device_reduce.hpp"
 
-template<
-    class Key,
-    class Value,
-    class ReduceOp,
-    unsigned int MinSegmentLength,
-    unsigned int MaxSegmentLength,
-    class Aggregate = Value
->
+#include "test_utils_data_generation.hpp"
+
+template<class Key,
+         class Value,
+         class ReduceOp,
+         unsigned int MinSegmentLength,
+         unsigned int MaxSegmentLength,
+         class Aggregate = Value,
+         bool UseGraphs  = false>
 struct params
 {
-    using key_type = Key;
-    using value_type = Value;
-    using reduce_op_type = ReduceOp;
+    using key_type                                   = Key;
+    using value_type                                 = Value;
+    using reduce_op_type                             = ReduceOp;
     static constexpr unsigned int min_segment_length = MinSegmentLength;
     static constexpr unsigned int max_segment_length = MaxSegmentLength;
-    using aggregate_type = Aggregate;
+    using aggregate_type                             = Aggregate;
+    static constexpr bool use_graphs                 = UseGraphs;
 };
 
 template<class Params>
@@ -62,33 +64,15 @@ typedef ::testing::Types<
     params<unsigned int, int, hipcub::Sum, 2048, 2048>,
     params<unsigned int, double, hipcub::Min, 1000, 50000>,
     params<long long, short, hipcub::Sum, 1000, 10000, long long>,
-    params<unsigned long long, unsigned long long, hipcub::Sum, 100000, 100000>
-#ifdef __HIP_PLATFORM_AMD__
-    ,
-    // Kernel doesn't work on NVidia.
+    params<unsigned long long, unsigned long long, hipcub::Sum, 100000, 100000>,
     // Sum for half and bfloat will result in values too big due to limited range.
     params<test_utils::half, test_utils::half, hipcub::Max, 3, 100>,
-    params<test_utils::bfloat16, test_utils::bfloat16, hipcub::Max, 20, 100>
-#endif
-    >
+    params<test_utils::bfloat16, test_utils::bfloat16, hipcub::Max, 20, 100>,
+    params<int, int, hipcub::Sum, 1, 1, int, true>>
     Params;
 
 TYPED_TEST_SUITE(HipcubDeviceReduceByKey, Params);
 
-std::vector<size_t> get_sizes()
-{
-    std::vector<size_t> sizes = {
-        1024, 2048, 4096, 1792,
-        1, 10, 53, 211, 500,
-        2345, 11001, 34567,
-        100000,
-        (1 << 16) - 1220, (1 << 23) - 76543
-    };
-    const std::vector<size_t> random_sizes = test_utils::get_random_data<size_t>(10, 1, 100000, rand());
-    sizes.insert(sizes.end(), random_sizes.begin(), random_sizes.end());
-    return sizes;
-}
-
 TYPED_TEST(HipcubDeviceReduceByKey, ReduceByKey)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -105,22 +89,25 @@ TYPED_TEST(HipcubDeviceReduceByKey, ReduceByKey)
         std::uniform_real_distribution<test_utils::convert_to_fundamental_t<key_type>>,
         std::uniform_int_distribution<test_utils::convert_to_fundamental_t<key_type>>>::type;
 
-    const bool debug_synchronous = false;
-
     reduce_op_type reduce_op;
     hipcub::Equality key_compare_op;
 
-    const std::vector<size_t> sizes = get_sizes();
-
-    for(size_t size : sizes)
+    hipStream_t stream = 0; // default
+    if(TestFixture::params::use_graphs)
     {
-        for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
-        {
-            unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
-            SCOPED_TRACE(testing::Message() << "with size = " << size);
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
-            hipStream_t stream = 0; // default
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        for(size_t size : test_utils::get_sizes(seed_value))
+        {
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
 
             // Generate data and calculate expected results
             std::vector<key_type> unique_expected;
@@ -206,32 +193,44 @@ TYPED_TEST(HipcubDeviceReduceByKey, ReduceByKey)
 
             size_t temporary_storage_bytes = 0;
 
-            HIP_CHECK(
-                hipcub::DeviceReduce::ReduceByKey(
-                    nullptr, temporary_storage_bytes,
-                    d_keys_input, d_unique_output,
-                    d_values_input, d_aggregates_output,
-                    d_unique_count_output,
-                    reduce_op, size,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceReduce::ReduceByKey(nullptr,
+                                                        temporary_storage_bytes,
+                                                        d_keys_input,
+                                                        d_unique_output,
+                                                        d_values_input,
+                                                        d_aggregates_output,
+                                                        d_unique_count_output,
+                                                        reduce_op,
+                                                        size,
+                                                        stream));
 
             ASSERT_GT(temporary_storage_bytes, 0U);
 
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            HIP_CHECK(
-                hipcub::DeviceReduce::ReduceByKey(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_keys_input, d_unique_output,
-                    d_values_input, d_aggregates_output,
-                    d_unique_count_output,
-                    reduce_op, size,
-                    stream, debug_synchronous
-                )
-            );
+            hipGraph_t graph;
+            if(TestFixture::params::use_graphs)
+            {
+                graph = test_utils::createGraphHelper(stream);
+            }
+
+            HIP_CHECK(hipcub::DeviceReduce::ReduceByKey(d_temporary_storage,
+                                                        temporary_storage_bytes,
+                                                        d_keys_input,
+                                                        d_unique_output,
+                                                        d_values_input,
+                                                        d_aggregates_output,
+                                                        d_unique_count_output,
+                                                        reduce_op,
+                                                        size,
+                                                        stream));
+
+            hipGraphExec_t graph_instance;
+            if(TestFixture::params::use_graphs)
+            {
+                graph_instance = test_utils::endCaptureGraphHelper(graph, stream, true, true);
+            }
 
             HIP_CHECK(hipFree(d_temporary_storage));
 
@@ -269,26 +268,22 @@ TYPED_TEST(HipcubDeviceReduceByKey, ReduceByKey)
             ASSERT_EQ(unique_count_output[0], unique_count_expected);
 
             // Validating results
-            for(size_t i = 0; i < unique_count_expected; i++)
-            {
-                ASSERT_EQ(test_utils::convert_to_native(unique_output[i]),
-                          test_utils::convert_to_native(unique_expected[i]));
+            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(unique_output, unique_expected));
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(aggregates_output,
+                                        aggregates_expected,
+                                        test_utils::precision<aggregate_type>::value
+                                            * TestFixture::params::max_segment_length));
 
-                if(std::is_integral<aggregate_type>::value)
-                {
-                    ASSERT_EQ(test_utils::convert_to_native(aggregates_output[i]),
-                              test_utils::convert_to_native(aggregates_expected[i]));
-                }
-                else if(test_utils::is_floating_point<aggregate_type>::value)
-                {
-                    auto tolerance = std::max<test_utils::convert_to_fundamental_t<aggregate_type>>(
-                        std::abs(0.1f * test_utils::convert_to_native(aggregates_expected[i])),
-                        aggregate_type(0.01f));
-                    ASSERT_NEAR(test_utils::convert_to_native(aggregates_output[i]),
-                                test_utils::convert_to_native(aggregates_expected[i]),
-                                tolerance);
-                }
+            if(TestFixture::params::use_graphs)
+            {
+                test_utils::cleanupGraphHelper(graph, graph_instance);
             }
         }
     }
+
+    if(TestFixture::params::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
