@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2020 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -26,27 +26,28 @@
 #include "hipcub/util_allocator.hpp"
 
 #include "common_test_header.hpp"
+#include "test_utils_assertions.hpp"
 
 hipcub::CachingDeviceAllocator g_allocator;
 
-static constexpr float alpha = 1.0f;
-static constexpr float beta = 0.0f;
+static constexpr float alpha_const = 1.0f;
+static constexpr float beta_const  = 0.0f;
 
 // Params for tests
-template<
-    class Type,
-    int32_t Grid2D = -1,
-    int32_t Grid3D = -1,
-    int32_t Wheel = -1,
-    int32_t Dense = -1
->
+template<class Type,
+         int32_t Grid2D    = -1,
+         int32_t Grid3D    = -1,
+         int32_t Wheel     = -1,
+         int32_t Dense     = -1,
+         bool    UseGraphs = false>
 struct DeviceSpmvParams
 {
-    using value_type = Type;
-    static constexpr int32_t grid_2d = Grid2D;
-    static constexpr int32_t grid_3d = Grid3D;
-    static constexpr int32_t wheel   = Wheel;
-    static constexpr int32_t dense   = Dense;
+    using value_type                    = Type;
+    static constexpr int32_t grid_2d    = Grid2D;
+    static constexpr int32_t grid_3d    = Grid3D;
+    static constexpr int32_t wheel      = Wheel;
+    static constexpr int32_t dense      = Dense;
+    static constexpr bool    use_graphs = UseGraphs;
 };
 
 // ---------------------------------------------------------
@@ -57,16 +58,17 @@ template<class Params>
 class HipcubDeviceSpmvTests : public ::testing::Test
 {
 public:
-    using value_type = typename Params::value_type;
-    static constexpr int32_t grid_2d = Params::grid_2d;
-    static constexpr int32_t grid_3d = Params::grid_3d;
-    static constexpr int32_t wheel   = Params::wheel;
-    static constexpr int32_t dense   = Params::dense;
+    using value_type                    = typename Params::value_type;
+    static constexpr int32_t grid_2d    = Params::grid_2d;
+    static constexpr int32_t grid_3d    = Params::grid_3d;
+    static constexpr int32_t wheel      = Params::wheel;
+    static constexpr int32_t dense      = Params::dense;
+    static constexpr bool    use_graphs = Params::use_graphs;
 };
 
-typedef ::testing::Types<
-    DeviceSpmvParams<float, 4, 0, 0, 0>
-> HipcubDeviceSpmvTestsParams;
+typedef ::testing::Types<DeviceSpmvParams<float, 4, 0, 0, 0>,
+                         DeviceSpmvParams<float, 4, 0, 0, 0, true>>
+    HipcubDeviceSpmvTestsParams;
 
 template<typename T, typename OffsetType>
 static void
@@ -110,8 +112,8 @@ template <
     typename OffsetType>
 void SpmvGold(
     CsrMatrix<T, OffsetType>&  a,
-    T*                         vector_x,
-    T*                         vector_y_in,
+    const T*                         vector_x,
+    const T*                         vector_y_in,
     T*                         vector_y_out,
     T                          alpha,
     T                          beta)
@@ -145,6 +147,11 @@ TYPED_TEST(HipcubDeviceSpmvTests, Spmv)
     constexpr int32_t dense   = TestFixture::dense;
 
     hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     CooMatrix<T, OffsetType> coo_matrix;
     generate_matrix(coo_matrix, grid_2d, grid_3d, wheel, dense);
@@ -165,7 +172,7 @@ TYPED_TEST(HipcubDeviceSpmvTests, Spmv)
         vector_y_in[row] = 1.0;
 
     // Compute reference answer
-    SpmvGold(csr_matrix, vector_x, vector_y_in, vector_y_out, alpha, beta);
+    SpmvGold(csr_matrix, vector_x, vector_y_in, vector_y_out, alpha_const, beta_const);
 
     // Allocate and initialize GPU problem
     hipcub::DeviceSpmv::SpmvParams<T, OffsetType> params;
@@ -179,8 +186,8 @@ TYPED_TEST(HipcubDeviceSpmvTests, Spmv)
     params.num_rows     = csr_matrix.num_rows;
     params.num_cols     = csr_matrix.num_cols;
     params.num_nonzeros = csr_matrix.num_nonzeros;
-    params.alpha        = alpha;
-    params.beta         = beta;
+    params.alpha        = alpha_const;
+    params.beta         = beta_const;
 
     HIP_CHECK(hipMemcpy(params.d_values,            csr_matrix.values,         sizeof(T) * csr_matrix.num_nonzeros, hipMemcpyHostToDevice));
     HIP_CHECK(hipMemcpy(params.d_row_end_offsets,   csr_matrix.row_offsets,    sizeof(OffsetType) * (csr_matrix.num_rows + 1), hipMemcpyHostToDevice));
@@ -193,49 +200,64 @@ TYPED_TEST(HipcubDeviceSpmvTests, Spmv)
     void *d_temp_storage = NULL;
 
     // Get amount of temporary storage needed
-    HIP_CHECK(hipcub::DeviceSpmv::CsrMV(
-                d_temp_storage,
-                temp_storage_bytes,
-                params.d_values,
-                params.d_row_end_offsets,
-                params.d_column_indices,
-                params.d_vector_x,
-                params.d_vector_y,
-                params.num_rows,
-                params.num_cols,
-                params.num_nonzeros,
-                stream,
-                false));
+    HIP_CHECK(hipcub::DeviceSpmv::CsrMV(d_temp_storage,
+                                        temp_storage_bytes,
+                                        params.d_values,
+                                        params.d_row_end_offsets,
+                                        params.d_column_indices,
+                                        params.d_vector_x,
+                                        params.d_vector_y,
+                                        params.num_rows,
+                                        params.num_cols,
+                                        params.num_nonzeros,
+                                        stream));
 
     // Allocate
     //HIP_CHECK(hipMalloc(&d_temp_storage, temp_storage_bytes);
     HIP_CHECK(g_allocator.DeviceAllocate(&d_temp_storage, temp_storage_bytes));
     HIP_CHECK(hipDeviceSynchronize());
 
-    HIP_CHECK(hipcub::DeviceSpmv::CsrMV(
-                d_temp_storage,
-                temp_storage_bytes,
-                params.d_values,
-                params.d_row_end_offsets,
-                params.d_column_indices,
-                params.d_vector_x,
-                params.d_vector_y,
-                params.num_rows,
-                params.num_cols,
-                params.num_nonzeros,
-                stream,
-                false));
+    hipGraph_t graph;
+    if(TestFixture::use_graphs)
+    {
+        graph = test_utils::createGraphHelper(stream);
+    }
+
+    HIP_CHECK(hipcub::DeviceSpmv::CsrMV(d_temp_storage,
+                                        temp_storage_bytes,
+                                        params.d_values,
+                                        params.d_row_end_offsets,
+                                        params.d_column_indices,
+                                        params.d_vector_x,
+                                        params.d_vector_y,
+                                        params.num_rows,
+                                        params.num_cols,
+                                        params.num_nonzeros,
+                                        stream));
+
+    hipGraphExec_t graph_instance;
+    if(TestFixture::use_graphs)
+    {
+        graph_instance = test_utils::endCaptureGraphHelper(graph, stream, true, true);
+    }
 
     HIP_CHECK(hipMemcpy(vector_y_in, params.d_vector_y, sizeof(T) * params.num_rows, hipMemcpyDeviceToHost));
 
     HIP_CHECK(hipPeekAtLastError());
     HIP_CHECK(hipDeviceSynchronize());
 
+    const auto  max_row_len = csr_matrix.num_cols * csr_matrix.num_rows;
+    const float diff        = max_row_len * test_utils::precision<T>::value;
+
     for(int32_t i = 0; i < csr_matrix.num_rows; i++)
     {
-        auto diff = std::max<T>(std::abs(0.01f * vector_y_out[i]), 0.01f);
-        ASSERT_NEAR(vector_y_in[i], vector_y_out[i], diff) << "where index = " << i;
+        ASSERT_NO_FATAL_FAILURE(test_utils::assert_near(vector_y_in[i], vector_y_out[i], diff))
+            << "where index = " << i;
     }
 
+    if(TestFixture::use_graphs)
+    {
+        test_utils::cleanupGraphHelper(graph, graph_instance);
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
-
