@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
 #include "common_test_header.hpp"
 
 #include "hipcub/warp/warp_reduce.hpp"
+#include <type_traits>
 
 template<
     class T,
@@ -63,6 +64,27 @@ typedef ::testing::Types<
 #ifdef __HIP_PLATFORM_AMD__
     params<float, 64U>,
 #endif
+    // half
+    params<test_utils::half, 1U>,
+    params<test_utils::half, 2U>,
+    params<test_utils::half, 4U>,
+    params<test_utils::half, 8U>,
+    params<test_utils::half, 16U>,
+    params<test_utils::half, 32U>,
+#ifdef __HIP_PLATFORM_AMD__
+    params<test_utils::half, 64U>,
+#endif
+    // bfloat16
+    params<test_utils::bfloat16, 1U>,
+    params<test_utils::bfloat16, 2U>,
+    params<test_utils::bfloat16, 4U>,
+    params<test_utils::bfloat16, 8U>,
+    params<test_utils::bfloat16, 16U>,
+    params<test_utils::bfloat16, 32U>,
+#ifdef __HIP_PLATFORM_AMD__
+    params<test_utils::bfloat16, 64U>,
+#endif
+
     // shared memory reduce
     // Integer
     params<int, 3U>,
@@ -86,14 +108,11 @@ typedef ::testing::Types<
 
 TYPED_TEST_SUITE(HipcubWarpReduceTests, HipcubWarpReduceTestParams);
 
-template<
-    class T,
-    unsigned int BlockSize,
-    unsigned int LogicalWarpSize
->
+template<class T, unsigned int BlockSize, unsigned int LogicalWarpSize>
 __global__
 __launch_bounds__(BlockSize)
-void warp_reduce_kernel(T* device_input, T* device_output)
+auto warp_reduce_kernel(T* device_input, T* device_output) ->
+    typename std::enable_if_t<test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
 {
     // Minimum size is 1
     constexpr unsigned int warps_no = test_utils::max(BlockSize / LogicalWarpSize, 1u);
@@ -113,6 +132,16 @@ void warp_reduce_kernel(T* device_input, T* device_output)
     }
 }
 
+template<class T, unsigned int BlockSize, unsigned int LogicalWarpSize>
+__global__
+__launch_bounds__(BlockSize)
+auto warp_reduce_kernel(T* /*device_input*/, T* /*device_output*/) ->
+    typename std::enable_if_t<!test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{
+    // This kernel should never be actually called; tests are filtered out at runtime
+    // if the device does not support the LogicalWarpSize
+}
+
 TYPED_TEST(HipcubWarpReduceTests, Reduce)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -120,6 +149,11 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     // logical warp side for warp primitive, execution warp size
     constexpr size_t logical_warp_size = TestFixture::warp_size;
 
@@ -128,16 +162,18 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
     constexpr size_t ws64 = size_t(HIPCUB_WARP_SIZE_64);
 
     // Block size of warp size 32
-    constexpr size_t block_size_ws32 =
-        test_utils::is_power_of_two(logical_warp_size)
-            ? test_utils::max<size_t>(ws32, logical_warp_size * 4)
-            : test_utils::max<size_t>((ws32/logical_warp_size) * logical_warp_size, 1);
+    constexpr size_t block_size_ws32
+        = test_utils::is_power_of_two(logical_warp_size)
+              ? test_utils::max<size_t>(ws32, logical_warp_size * 4)
+              : test_utils::max<size_t>((ws32 / logical_warp_size) * logical_warp_size,
+                                        static_cast<size_t>(1));
 
     // Block size of warp size 64
-    constexpr size_t block_size_ws64 =
-        test_utils::is_power_of_two(logical_warp_size)
-            ? test_utils::max<size_t>(ws64, logical_warp_size * 4)
-            : test_utils::max<size_t>((ws64/logical_warp_size) * logical_warp_size, 1);
+    constexpr size_t block_size_ws64
+        = test_utils::is_power_of_two(logical_warp_size)
+              ? test_utils::max<size_t>(ws64, logical_warp_size * 4)
+              : test_utils::max<size_t>((ws64 / logical_warp_size) * logical_warp_size,
+                                        static_cast<size_t>(1));
 
     const unsigned int current_device_warp_size = HIPCUB_HOST_WARP_THREADS;
 
@@ -149,7 +185,7 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -160,20 +196,20 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
         SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
 
         // Generate data
-        std::vector<T> input = test_utils::get_random_data<T>(size, -100, 100, seed_value);
-        std::vector<T> output(size / logical_warp_size, 0);
-        std::vector<T> expected(output.size(), 1);
+        std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
+        std::vector<T> output(size / logical_warp_size);
+        std::vector<T> expected(output.size());
 
         // Calculate expected results on host
         for(size_t i = 0; i < output.size(); i++)
         {
-            T value = 0;
+            acc_type value(0);
             for(size_t j = 0; j < logical_warp_size; j++)
             {
                 auto idx = i * logical_warp_size + j;
-                value += input[idx];
+                value    = binary_op_host(input[idx], value);
             }
-            expected[i] = value;
+            expected[i] = static_cast<T>(value);
         }
 
         // Writing to device memory
@@ -220,26 +256,20 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
             )
         );
 
-        for(size_t i = 0; i < output.size(); i++)
-        {
-            auto diff = std::max<T>(std::abs(0.1f * expected[i]), T(0.01f));
-            if(std::is_integral<T>::value) diff = 0;
-            ASSERT_NEAR(output[i], expected[i], diff) << "where index = " << i;
-        }
+        test_utils::assert_near(output,
+                                expected,
+                                test_utils::precision<T>::value * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
     }
 }
 
-template<
-    class T,
-    unsigned int BlockSize,
-    unsigned int LogicalWarpSize
->
+template<class T, unsigned int BlockSize, unsigned int LogicalWarpSize>
 __global__
 __launch_bounds__(BlockSize)
-void warp_reduce_valid_kernel(T* device_input, T* device_output, const int valid)
+auto warp_reduce_valid_kernel(T* device_input, T* device_output, const int valid) ->
+    typename std::enable_if_t<test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
 {
     // Minimum size is 1
     constexpr unsigned int warps_no = test_utils::max(BlockSize / LogicalWarpSize, 1u);
@@ -259,6 +289,16 @@ void warp_reduce_valid_kernel(T* device_input, T* device_output, const int valid
     }
 }
 
+template<class T, unsigned int BlockSize, unsigned int LogicalWarpSize>
+__global__
+__launch_bounds__(BlockSize)
+auto warp_reduce_valid_kernel(T* /*device_input*/, T* /*device_output*/, const int /*valid*/) ->
+    typename std::enable_if_t<!test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{
+    // This kernel should never be actually called; tests are filtered out at runtime
+    // if the device does not support the LogicalWarpSize
+}
+
 TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -266,6 +306,11 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     // logical warp side for warp primitive, execution warp size
     constexpr size_t logical_warp_size = TestFixture::warp_size;
     // The different warp sizes
@@ -273,16 +318,18 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
     constexpr size_t ws64 = size_t(HIPCUB_WARP_SIZE_64);
 
     // Block size of warp size 32
-    constexpr size_t block_size_ws32 =
-        test_utils::is_power_of_two(logical_warp_size)
-            ? test_utils::max<size_t>(ws32, logical_warp_size * 4)
-            : test_utils::max<size_t>((ws32/logical_warp_size) * logical_warp_size, 1);
+    constexpr size_t block_size_ws32
+        = test_utils::is_power_of_two(logical_warp_size)
+              ? test_utils::max<size_t>(ws32, logical_warp_size * 4)
+              : test_utils::max<size_t>((ws32 / logical_warp_size) * logical_warp_size,
+                                        static_cast<size_t>(1));
 
     // Block size of warp size 64
-    constexpr size_t block_size_ws64 =
-        test_utils::is_power_of_two(logical_warp_size)
-            ? test_utils::max<size_t>(ws64, logical_warp_size * 4)
-            : test_utils::max<size_t>((ws64/logical_warp_size) * logical_warp_size, 1);
+    constexpr size_t block_size_ws64
+        = test_utils::is_power_of_two(logical_warp_size)
+              ? test_utils::max<size_t>(ws64, logical_warp_size * 4)
+              : test_utils::max<size_t>((ws64 / logical_warp_size) * logical_warp_size,
+                                        static_cast<size_t>(1));
 
     const unsigned int current_device_warp_size = HIPCUB_HOST_WARP_THREADS;
 
@@ -295,7 +342,7 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -306,20 +353,20 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
         SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
 
         // Generate data
-        std::vector<T> input = test_utils::get_random_data<T>(size, -100, 100, seed_value);
-        std::vector<T> output(size / logical_warp_size, 0);
-        std::vector<T> expected(output.size(), 1);
+        std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
+        std::vector<T> output(size / logical_warp_size);
+        std::vector<T> expected(output.size());
 
         // Calculate expected results on host
         for(size_t i = 0; i < output.size(); i++)
         {
-            T value = 0;
-            for(size_t j = 0; j < valid; j++)
+            acc_type value(0);
+            for(int j = 0; j < valid; ++j)
             {
                 auto idx = i * logical_warp_size + j;
-                value += input[idx];
+                value    = binary_op_host(input[idx], value);
             }
-            expected[i] = valid ? value : input[i];
+            expected[i] = valid ? static_cast<T>(value) : input[i];
         }
 
         // Writing to device memory
@@ -366,27 +413,20 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
             )
         );
 
-        for(size_t i = 0; i < output.size(); i++)
-        {
-            auto diff = std::max<T>(std::abs(0.1f * expected[i]), T(0.01f));
-            if(std::is_integral<T>::value) diff = 0;
-            ASSERT_NEAR(output[i], expected[i], diff) << "where index = " << i;
-        }
+        test_utils::assert_near(output,
+                                expected,
+                                test_utils::precision<T>::value * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
     }
 }
 
-template<
-    class T,
-    class Flag,
-    unsigned int BlockSize,
-    unsigned int LogicalWarpSize
->
+template<class T, class Flag, unsigned int BlockSize, unsigned int LogicalWarpSize>
 __global__
 __launch_bounds__(BlockSize)
-void head_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output)
+auto head_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output) ->
+    typename std::enable_if_t<test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
 {
     // Minimum size is 1
     constexpr unsigned int warps_no = test_utils::max(BlockSize / LogicalWarpSize, 1u);
@@ -403,6 +443,16 @@ void head_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output)
     output[index] = value;
 }
 
+template<class T, class Flag, unsigned int BlockSize, unsigned int LogicalWarpSize>
+__global__
+__launch_bounds__(BlockSize)
+auto head_segmented_warp_reduce_kernel(T* /*input*/, Flag* /*flags*/, T* /*output*/) ->
+    typename std::enable_if_t<!test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{
+    // This kernel should never be actually called; tests are filtered out at runtime
+    // if the device does not support the LogicalWarpSize
+}
+
 TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -410,6 +460,11 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     using flag_type = unsigned char;
     // logical warp side for warp primitive, execution warp size
     constexpr size_t logical_warp_size = TestFixture::warp_size;
@@ -419,16 +474,18 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
     constexpr size_t ws64 = size_t(HIPCUB_WARP_SIZE_64);
 
     // Block size of warp size 32
-    constexpr size_t block_size_ws32 =
-        test_utils::is_power_of_two(logical_warp_size)
-            ? test_utils::max<size_t>(ws32, logical_warp_size * 4)
-            : test_utils::max<size_t>((ws32/logical_warp_size) * logical_warp_size, 1);
+    constexpr size_t block_size_ws32
+        = test_utils::is_power_of_two(logical_warp_size)
+              ? test_utils::max<size_t>(ws32, logical_warp_size * 4)
+              : test_utils::max<size_t>((ws32 / logical_warp_size) * logical_warp_size,
+                                        static_cast<size_t>(1));
 
     // Block size of warp size 64
-    constexpr size_t block_size_ws64 =
-        test_utils::is_power_of_two(logical_warp_size)
-            ? test_utils::max<size_t>(ws64, logical_warp_size * 4)
-            : test_utils::max<size_t>((ws64/logical_warp_size) * logical_warp_size, 1);
+    constexpr size_t block_size_ws64
+        = test_utils::is_power_of_two(logical_warp_size)
+              ? test_utils::max<size_t>(ws64, logical_warp_size * 4)
+              : test_utils::max<size_t>((ws64 / logical_warp_size) * logical_warp_size,
+                                        static_cast<size_t>(1));
 
     const unsigned int current_device_warp_size = HIPCUB_HOST_WARP_THREADS;
 
@@ -440,7 +497,7 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -503,21 +560,21 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
         // Calculate expected results on host
         std::vector<T> expected(output.size());
         size_t segment_head_index = 0;
-        T reduction = input[0];
+        acc_type       reduction(input[0]);
         for(size_t i = 0; i < output.size(); i++)
         {
             if(i%logical_warp_size == 0 || flags[i])
             {
-                expected[segment_head_index] = reduction;
+                expected[segment_head_index] = static_cast<T>(reduction);
                 segment_head_index = i;
                 reduction = input[i];
             }
             else
             {
-                reduction = reduction + input[i];
+                reduction = binary_op_host(input[i], reduction);
             }
         }
-        expected[segment_head_index] = reduction;
+        expected[segment_head_index] = static_cast<T>(reduction);
 
         // Launching kernel
         if (current_device_warp_size == ws32)
@@ -554,15 +611,19 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
         );
         HIP_CHECK(hipDeviceSynchronize());
 
+        std::vector<T> output_segment(output.size(), T(0));
+        std::vector<T> expected_segment(output.size(), T(0));
         for(size_t i = 0; i < output.size(); i++)
         {
             if(flags[i])
             {
-                auto diff = std::max<T>(std::abs(0.1f * expected[i]), T(0.01f));
-                if(std::is_integral<T>::value) diff = 0;
-                ASSERT_NEAR(output[i], expected[i], diff) << "where index = " << i;
+                output_segment[i]   = output[i];
+                expected_segment[i] = expected[i];
             }
         }
+        test_utils::assert_near(output_segment,
+                                expected_segment,
+                                test_utils::precision<T>::value * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_flags));
@@ -570,15 +631,11 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
     }
 }
 
-template<
-    class T,
-    class Flag,
-    unsigned int BlockSize,
-    unsigned int LogicalWarpSize
->
+template<class T, class Flag, unsigned int BlockSize, unsigned int LogicalWarpSize>
 __global__
 __launch_bounds__(BlockSize)
-void tail_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output)
+auto tail_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output) ->
+    typename std::enable_if_t<test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
 {
     // Minimum size is 1
     constexpr unsigned int warps_no = test_utils::max(BlockSize / LogicalWarpSize, 1u);
@@ -596,6 +653,16 @@ void tail_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output)
     output[index] = value;
 }
 
+template<class T, class Flag, unsigned int BlockSize, unsigned int LogicalWarpSize>
+__global__
+__launch_bounds__(BlockSize)
+auto tail_segmented_warp_reduce_kernel(T* /*input*/, Flag* /*flags*/, T* /*output*/) ->
+    typename std::enable_if_t<!test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{
+    // This kernel should never be actually called; tests are filtered out at runtime
+    // if the device does not support the LogicalWarpSize
+}
+
 TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -603,6 +670,11 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     using flag_type = unsigned char;
     // logical warp side for warp primitive, execution warp size
     constexpr size_t logical_warp_size = TestFixture::warp_size;
@@ -612,16 +684,18 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
     constexpr size_t ws64 = size_t(HIPCUB_WARP_SIZE_64);
 
     // Block size of warp size 32
-    constexpr size_t block_size_ws32 =
-        test_utils::is_power_of_two(logical_warp_size)
-            ? test_utils::max<size_t>(ws32, logical_warp_size * 4)
-            : test_utils::max<size_t>((ws32/logical_warp_size) * logical_warp_size, 1);
+    constexpr size_t block_size_ws32
+        = test_utils::is_power_of_two(logical_warp_size)
+              ? test_utils::max<size_t>(ws32, logical_warp_size * 4)
+              : test_utils::max<size_t>((ws32 / logical_warp_size) * logical_warp_size,
+                                        static_cast<size_t>(1));
 
     // Block size of warp size 64
-    constexpr size_t block_size_ws64 =
-        test_utils::is_power_of_two(logical_warp_size)
-            ? test_utils::max<size_t>(ws64, logical_warp_size * 4)
-            : test_utils::max<size_t>((ws64/logical_warp_size) * logical_warp_size, 1);
+    constexpr size_t block_size_ws64
+        = test_utils::is_power_of_two(logical_warp_size)
+              ? test_utils::max<size_t>(ws64, logical_warp_size * 4)
+              : test_utils::max<size_t>((ws64 / logical_warp_size) * logical_warp_size,
+                                        static_cast<size_t>(1));
 
     const unsigned int current_device_warp_size = HIPCUB_HOST_WARP_THREADS;
 
@@ -633,7 +707,7 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -693,10 +767,11 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
         HIP_CHECK(hipDeviceSynchronize());
 
         // Calculate expected results on host
-        std::vector<T> expected(output.size());
+        std::vector<T>      expected(output.size());
         std::vector<size_t> segment_indexes;
-        size_t segment_index = 0;
-        T reduction;
+        size_t              segment_index = 0;
+        acc_type            accumulator(0);
+        acc_type            reduction(0);
         for(size_t i = 0; i < output.size(); i++)
         {
             // single value segments
@@ -704,20 +779,20 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
             {
                 expected[i] = input[i];
                 segment_indexes.push_back(i);
-            }
-            else
+            } else
             {
                 segment_index = i;
-                reduction = input[i];
-                auto next = i + 1;
+                reduction     = input[i];
+                auto next     = i + 1;
                 while(next < output.size() && !flags[next])
                 {
-                    reduction = reduction + input[next];
+                    reduction = binary_op_host(input[next], reduction);
                     i++;
                     next++;
                 }
                 i++;
-                expected[segment_index] = reduction + input[i];
+                accumulator             = binary_op_host(reduction, input[i]);
+                expected[segment_index] = static_cast<T>(accumulator);
                 segment_indexes.push_back(segment_index);
             }
         }
@@ -757,13 +832,17 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
         );
         HIP_CHECK(hipDeviceSynchronize());
 
+        std::vector<T> output_segment(segment_indexes.size());
+        std::vector<T> expected_segment(segment_indexes.size());
         for(size_t i = 0; i < segment_indexes.size(); i++)
         {
             auto index = segment_indexes[i];
-            auto diff = std::max<T>(std::abs(0.1f * expected[i]), T(0.01f));
-            if(std::is_integral<T>::value) diff = 0;
-            ASSERT_NEAR(output[index], expected[index], diff) << "where index = " << i;
+            output_segment[i]   = output[index];
+            expected_segment[i] = expected[index];
         }
+        test_utils::assert_near(output_segment,
+                                expected_segment,
+                                test_utils::precision<T>::value * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_flags));
